/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * External.h - modulo a ser incluido em qualquer implementacao externa ao
 *              GeoLab.
 *
 * Importante:
 *             Se estiver sendo criada alguma classe nova (descendente de Obj), 
 *             nao esquecer de colocar Id em ExternalObj_id.h (que deve ser 
 *             incluido no .h da classe).
 *
 *	       Alem deste header, todo algoritmo externo deve importar o
 *	       arquivo correspondente `a definicao da superclasse do algoritmo.
 *             (e.g., Algorithm.h no caso de descendencia direta).
 *
 * 
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _EXTERNAL_DEFINED
#define _EXTERNAL_DEFINED

/*
 * Nota do Autor:
 *
 *    Uma vez que o ambiente esteja estabilizado e aceitando novos algoritmos
 *    regularmente, sugere-se que os 'includes` presentes neste modulo sejam
 *    retirados e substituidos pelo corpo dos modulos, de forma a facilitar
 *    seu uso, inclusive no que diz respeito a Makefile's. No momento nao esta'
 *    feito assim devido a constantes modificacoes a que sao sujeitos tais
 *    modulos e portanto torna-se inviavel manter consistencia.
 *
 */
 
#include <xview/xview.h>
#include <xview/cursor.h>
#include "global.h"
#include "basic.h"		// os modulos incluidos aqui (separados dos
#include "tools.h"		// demais) sao obrigatorios em todo modulo
#include "utilities.h"		// externo a ser ligado dinamicamente
#include "List.h"
#include "World.h"
#include "String.h"
#include "Obj.h"
#include "Algorithm.h"
#include "ExternalObj_id.h"	

#ifndef _Obj_ptr_list
#define _Obj_ptr_list
declare(list,Obj_ptr)
#endif ~_Obj_ptr_list;

#ifndef _GraphicObj_ptr_list
#define _GraphicObj_ptr_list
declare(list,GraphicObj_ptr)
#endif ~_GraphicObj_ptr_list;

#ifndef _list_item_list
#define _list_item_list
declare(list,list_item)
#endif ~_list_item_list;

typedef list(Obj_ptr) ResultObjects;
typedef list(GraphicObj_ptr) GraphicObjects;
typedef list(list_item) SelectedObjects;

// funcoes extras para possibilitar o uso, por parte de algoritmos externos,
// das facilidades de "extra object" e "extra mode" (para, entre outras
// coisas, tornar algoritmos externos interativos passiveis de implementacao)

extern void exec_binder(World *, Algorithm *);
extern void EF_exec_binder(World *);
extern void EO_exec_binder(World *);
extern void install_external_function_mode(World *, ExternalFunctionMode *);
extern void uninstall_external_function_mode(World *);
extern void install_external_object_mode(World *, ExternalObjectMode *);
extern void uninstall_external_object_mode(World *);

// facilidades para manipula,c~ao da base de dados de objetos geometricos do GeoLab
GraphicObjects& objects_get_graphic_objects();
SelectedObjects& objects_get_selected_objects();
list(Obj_ptr) extract_selected_objects(ObjType type, ObjType subtype);
list(Obj_ptr) extract_objects(ObjType type, ObjType subtype);
list(GraphicObj_ptr) extract_selected_graphic_objects(ObjType type, ObjType subtype);
list(GraphicObj_ptr) extract_graphic_objects(ObjType type, ObjType subtype);
void objects_refresh(World *w);
void objects_repaint(World *w);
void objects_repaint_all_but(GraphicObj *not_to_repaint, World *w);

// insercao de novos objetos
extern void install_externally_created_object(GraphicObj_ptr, Algorithm *);
extern void EF_install_externally_created_object(GraphicObj_ptr);
extern void EO_install_externally_created_object(GraphicObj_ptr);

// facilidades para tratamentos de eventos
extern bool compress_drag_events(World *, Xv_Window, Event *);
extern bool fixed_movement_events(World *, Xv_Window, Event *);
extern bool fixed_movement_events_points(World *, Xv_Window, Event *);

// facilidades para manipulacao de cursores
void set_cursor(World *w, Xv_Cursor cursor);

EXPORT Xv_Cursor insert_cursor;
EXPORT Xv_Cursor select_cursor;
EXPORT Xv_Cursor move_cursor;
EXPORT Xv_Cursor copy_cursor;
EXPORT Xv_Cursor resizeop_cursor;
EXPORT Xv_Cursor resizecl_cursor;
EXPORT Xv_Cursor zoom_cursor;
EXPORT Xv_Cursor scroll_cursor;
EXPORT Xv_Cursor dynamic_cursor;

// interligacao com interface (cor, tipo de linha, etc)

EXPORT unsigned long FOREGROUND;
EXPORT unsigned long BACKGROUND;

EXPORT int LINE_WIDTH;
EXPORT int LINE_STYLE;

EXPORT bool HandlesOn;

// funcoes para animacao

inline bool animation(int animation_depth, int reference_depth) {
    return reference_depth && reference_depth >= animation_depth;
}

#define ANIMATION_BEGIN(w,anim_depth,ref_depth) \
if (::animation(anim_depth, ref_depth)) {		\
  w->Flush();
  
#define ANIMATION_END(w,anim_depth,ref_depth)	\
  w->Flush();					\
}

static bool __nhaca__ = true;
#define ANIMATION_CODE(w,anim_depth,ref_depth) \
for (__nhaca__ =  true;__nhaca__ && (::animation(anim_depth, ref_depth) ? (w->Flush(),true) : false); w->Flush(), __nhaca__=false)

extern void frame_stop_mark(World *);
extern void frame_delay_mark(World *);

// suporte para cores

#define BLACK		0
#define WHITE		15

extern unsigned long FOREGROUND;
extern unsigned long BACKGROUND;

extern unsigned long get_pixel_from_value(int value);

#endif ~_EXTERNAL_DEFINED;
