/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Line2D.h - definicao das classes Line2D e GraphicLine2D.
 *
 *            link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _LINE2D_DEFINED
#define _LINE2D_DEFINED

#include <iostream.h>
#include <stdio.h>
#include <math.h>
#include "global.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"

// forward declaration
class GraphicLine2D;

class Line2D : public Obj {
  public:
    // construtores
    Line2D(char *s = "Line2D", ObjType t = Generic_t, bool v = false) : Obj(s, t, v) { set(0,0,0,1); }
    Line2D(double x1, double y1, double x2, double y2) : Obj("Line2D") { set(x1, y1,x2, y2); }
    Line2D( Point2D &a,  Point2D &b) : Obj("Line2D") { set(a,b); }
    Line2D( Line2D &l) : Obj(l.what_is(), l.stype, l.volati) { set(l.P1(), l.P2()); }
    
    //destrutor
    ~Line2D() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                    Line2D --> GraphicLine2D
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Line2D(*this);
    }
        
    // query & set functions
    Point2D P1()  { return p1; }
    Point2D P2()  { return p2; }    

    virtual void set(double x1, double y1, double x2, double y2) {
    	if (x1 == x2 && y1 == y2)
    	  x2 = x1 + 1;
    	p1.setXY(x1, y1);	// pontos devem ser considerados
    	p2.setXY(x2, y2);	// como estando no infinito
        x0 = x1; y0 = y1;
        f = x2 - x1;
        g = y2 - y1;
    }
    
    virtual void set( Point2D &a,  Point2D &b) {
    	int aux = (a == b) ? 1 : 0;
        p1.setXY(a.X(), a.Y()); p2.setXY(b.X(), b.Y());
        x0 = p1.X(); y0 = p1.Y();
        f = p2.X() - p1.X() + aux;
        g = p2.Y() - p1.Y();
    }
    
    bool equal( Line2D &l)  { return (p1 == l.p1 && p2 == l.p2) || p1 == l.p2 && p2 == l.p1; }
    bool operator == ( Line2D &l)  { return equal(l); }

    Point2D point_at(double t)  {
    	return Point2D(x0 + ( f * t), y0 + (g * t));
    }
        
    bool vertical()  { return f == 0; }
    bool horizontal()  { return g == 0; }

    // geometric functions
    bool cross_to_left( Line2D &l) ;
    bool cross_to_right( Line2D &l) ;
    bool intersect( Line2D &l) ;
    bool intersect( Point2D &a,  Point2D &b) ;
    Point2D intersection( Line2D &l) ;
    Point2D intersection( Point2D &a,  Point2D &b) ;
    Line2D normal_through_point( Point2D &p) ;
    IntersectType circle_intersections( Point2D &center, double radius, Point2D &i1, Point2D &i2) ;
    
    // inherited methods redefinition
    void load(FILE *f) { Obj::load(f); p1.load(f); p2.load(f); set(p1, p2); }
    void save(FILE *f) { Obj::save(f); p1.save(f); p2.save(f); }

    ObjType type()  { return Line2D_t; }
    
    void translate(double to_x, double to_y) {
    	p1.translate(to_x, to_y);
    	p2.translate(to_x, to_y);
    	set(p1, p2);
    }

    void rotate(double orig_x, double orig_y, double angle) {
    	p1.rotate(orig_x, orig_y, angle);
    	p2.rotate(orig_x, orig_y, angle);
    	set(p1, p2);
    }

    
    void print()  { cout << "Line2D ... P1 (" << P1().X() << "," << P1().Y() << ")" << " to P2 (" << P2().X() << "," << P2().Y() << ")"<< endl; }

    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *w, GC gc) ;

    bool select_test(double x, double y)  {
    	return p1.select_test(x, y) || p2.select_test(x, y);
    }
    
    bool select_test(double x1, double y1, double x2, double y2)  {
    	return p1.select_test(x1, y1, x2, y2) && p2.select_test(x1, y1, x2, y2);
    }

    double distance_to_point(double x, double y) ;

    void translate_handle_at(double x, double y, double to_x, double to_y);

  protected:
    double x0, y0;			// equacao parametrica da reta
    double f, g;
    Point2D p1, p2;			// handles
};

/*
 *
 * GraphicLine2D.h
 *
 *
 */

class GraphicLine2D : public GraphicObj {
  public:
    // construtores
    GraphicLine2D() : GraphicObj(new Line2D()) {}
    GraphicLine2D(Line2D *l) : GraphicObj(l) {}
    
    //destrutor
    ~GraphicLine2D() {}				
    
    // inherited methods redefinition
    void int_start(World *, double initial_X, double initial_Y, Int_Mode);
    void int_change(World *, double new_X, double new_Y, Int_Mode);
    void int_end(World *, double final_X, double final_Y, Int_Mode);
    
    void resize_start(World *, double X, double Y, Int_Mode);
    void resize_change(World *, double X, double Y, Int_Mode);
    void resize_end(World *, double X, double Y, Int_Mode);
    bool where_draw_string (double &x, double &y);
};

typedef Line2D* Line2D_ptr;
typedef GraphicLine2D* GraphicLine2D_ptr;

#endif ~_LINE2D_DEFINED;









