/*
 *
 * Ray2D.h - definicao das classes Ray2D e GraphicRay2D.
 *
 *           link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _RAY2D_DEFINED
#define _RAY2D_DEFINED

#include <iostream.h>
#include <math.h>
#include "global.h"
#include "World.h"
#include "Line2D.h"

// forward declaration
class GraphicRay2D;

class Ray2D : public Line2D {
  public:
    // construtores
    Ray2D(char *s = "Ray2D", ObjType t = Generic_t, bool v = false) : Line2D(s, t, v) { set(0,0,0,1); }
    Ray2D(double x1, double y1, double x2, double y2) : Line2D("Ray2D") { set(x1, y1, x2, y2); }
    Ray2D( Point2D &a,  Point2D &b) : Line2D("Ray2D") { set(a, b); }
    Ray2D( Ray2D &r) : Line2D(r.what_is(), r.stype, r.volati) { set(r.P1(), r.P2()); }

    // Obs: p2 deve ser considerado como estando no infinito
        
    //destrutor
    ~Ray2D() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                      Ray2D --> GraphicRay2D
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Ray2D(*this);
    }
        
    // geometric functions
    bool intersect( Ray2D &r) ;
    bool intersect( Line2D &l) ;
    bool intersect( Point2D &a,  Point2D &b) ;
    Point2D intersection( Ray2D &r) ;
    Point2D intersection( Line2D &l) ;
    Point2D intersection( Point2D &a,  Point2D &b) ;
    IntersectType circle_intersections( Point2D &center, double radius, Point2D &i1, Point2D &i2) ;

    ObjType type()  { return Ray2D_t; }
    
    void print()  { cout << "Ray2D ... P1 (" << P1().X() << "," << P1().Y() << ")" << " to P2 (" << P2().X() << "," << P2().Y() << ")"<< endl; }

    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *w, GC gc) ;

    double distance_to_point(double x, double y) ;
};

/*
 *
 * GraphicRay2D.h
 *
 *
 */

class GraphicRay2D : public GraphicLine2D {
  public:
    // construtores
    GraphicRay2D() : GraphicLine2D(new Ray2D()) {}
    GraphicRay2D(Ray2D *r) : GraphicLine2D(r) {}
    
    //destrutor
    ~GraphicRay2D() {}
};

typedef Ray2D* Ray2D_ptr;
typedef GraphicRay2D* GraphicRay2D_ptr;

#endif ~_RAY2D_DEFINED;







