/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Arrow2D.h - definicao das classes Arrow2D e GraphicArrow2D.
 *
 *             link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _ARROW2D_DEFINED
#define _ARROW2D_DEFINED

#include <iostream.h>
#include "global.h"
#include "tools.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "ExternalObj_id.h"

// forward declaration
class GraphicArrow2D;

class Arrow2D : public Segment2D {
  public:
    // construtores
    Arrow2D(char *s = "Arrow2D", ObjType t = Generic_t, bool v = false) : Segment2D(s, t, v) {}
    Arrow2D(double x1, double y1, double x2, double y2) : Segment2D("Arrow2D") { set(x1, y1, x2, y2); }
    Arrow2D( Point2D &a,  Point2D &b) : Segment2D("Arrow2D") { set(a, b); }
    Arrow2D(Arrow2D &a) : Segment2D(a.what_is(), a.stype, a.volati) { set(a.P1(), a.P2()); }
    
    //destrutor
    ~Arrow2D() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                  Arrow2D --> GraphicArrow2D
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Arrow2D(*this);
    }
        
    // inherited methods redefinition
    ObjType type()  { return Arrow2D_t; }
    
    void print()  { cout << "Arrow2D ... P1 (" << P1().X() << "," << P1().Y() << ")" << " to P2 (" << P2().X() << "," << P2().Y() << ")"<< endl; }

    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *w, GC gc) ;
};

/*
 *
 * GraphicArrow2D.h
 *
 *
 */

class GraphicArrow2D : public GraphicSegment2D {
  public:
    // construtores
    GraphicArrow2D() : GraphicSegment2D(new Arrow2D()) {}
    GraphicArrow2D(Arrow2D *a) : GraphicSegment2D(a) {}
    
    //destrutor
    ~GraphicArrow2D() {}				
};

typedef Arrow2D* Arrow2D_ptr;
typedef GraphicArrow2D* GraphicArrow2D_ptr;

#endif ~_Arrow2D_DEFINED;






