/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Circle.h - definicao das classes Circle e GraphicCircle.
 *
 *            link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _CIRCLE_DEFINED
#define _CIRCLE_DEFINED

#include <iostream.h>
#include <stdio.h>
#include <math.h>
#include "global.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"
#include "Square.h"

// forward declaration
class GraphicCircle;

class Circle : public Obj {
  public:
    // construtores
    Circle(char *s = "Circle", ObjType t = Generic_t, bool v = false) : Obj(s, t, v) { set(0, 0, 0); }
    Circle(double x, double y, double radius) : Obj("Circle") { set(x, y, radius); }
    Circle(double cx, double cy, double bx, double by) : Obj("Circle") { set(cx, cy, bx, by); }
    Circle( Point2D &center, double radius) : Obj("Circle") { set(center, radius); }
    Circle( Point2D &center,  Point2D &boundary) : Obj("Circle") { set(center, boundary); }
    Circle( Circle &c) : Obj(c.what_is(), c.stype, c.volati) { set(c.center(), c.radius()); }    
    
    //destrutor
    ~Circle() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                 Circle --> GraphicCircle
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Circle(*this);
    }

    // Metodos necessarios para "Input Tools"
    
    virtual bool point_inside_implemented () {return true;}
    virtual bool point_inside(double x, double y) ;
    virtual bool bounding_box_implemented () {return true;}
    virtual void bounding_box (double& x1, double& y1, double& x2, double& y2) ;
    virtual bool get_segments_implemented () {return true;}
    virtual list(Obj_ptr) get_segments () ;        
    
    // query & set functions
    Point2D& center() { return c; }
    double&  radius() { return r; }

    virtual void set_center(double cx, double cy) { c.setXY(cx, cy); }
    virtual void set_center( Point2D &center) { c = center; }
    virtual void set_radius(double radius) { r = radius; }
    virtual void set_boundary(double bx, double by) { 
      Point2D boundary(bx, by);
      set_radius(c.e_distance(boundary));
    }
    	
    virtual void set(double x, double y, double radius) { set_center(x, y); set_radius(radius); }

    virtual void set(double cx, double cy, double bx, double by) {
    	set_center(cx, cy);
    	set_boundary(bx, by);
    }
    
    virtual void set( Point2D &center, double radius) { set_center(center); set_radius(radius); }

    virtual void set( Point2D& center,  Point2D &boundary) {
    	double radius = center.e_distance(boundary);
    	set(center, radius);
    }

    virtual void set_inscr( Square &s) {
      set((s.P1().X() + s.P2().X()) / 2, (s.P1().Y() + s.P2().Y()) / 2, s.width() / 2);
    }
    
    virtual void set_circum( Square &s) {
      set((s.P1().X() + s.P2().X()) / 2, (s.P1().Y() + s.P2().Y()) / 2, sqrt (2 * ((s.width() / 2) * (s.width() / 2))));
    }
    
    bool equal( Circle &circle)  {
    	bool same_c = (circle.center() == center());
        return same_c && ::equal(circle.radius(), radius());
    }

    bool operator == ( Circle &circle)  { return equal(circle); }
    
    // geometric functions

    bool unit_circle()  { return ::equal(radius(), 1); }
    bool inside( Point2D &p)  { return p.e_distance(c) <= r; }
    
    // inherited methods redefinition        
    void load(FILE *f);
    void save(FILE *f);

    ObjType type()  { return Circle_t; }
    
    void translate(double to_x, double to_y) {
    	c.translate(to_x, to_y);
    }

    void rotate(double orig_x, double orig_y, double angle) {
	c.rotate(orig_x, orig_y, angle);
    }
    
    void print()  { cout << "Circle with center at (" << c.X() << "," << c.Y() << ") and radius " << r << endl; }

    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *w, GC gc) ;

    bool select_test(double x, double y)  {
    	set_handles();
    	return h1.select_test(x, y) || h2.select_test(x, y) || h3.select_test(x, y) || h4.select_test(x, y); 
    }
    
    bool select_test(double x1, double y1, double x2, double y2)  {
    	set_handles();
    	return h1.select_test(x1, y1, x2, y2) && h2.select_test(x1, y1, x2, y2) && h3.select_test(x1, y1, x2, y2) && h4.select_test(x1, y1, x2, y2); 
    }

    double distance_to_point(double x, double y)  {
    	double dc = c.distance_to_point(x, y);
    	if (dc < r)
    	  return r - dc;
    	else
    	  return dc - r;
    }
        	
  protected:
    void set_handles() {
        h1.setXY(c.X() - r, c.Y() - r);
        h2.setXY(c.X() - r, c.Y() + r);
        h3.setXY(c.X() + r, c.Y() - r);
        h4.setXY(c.X() + r, c.Y() + r);
    }
    
    double r;
    Point2D c;
    Point2D h1, h2, h3, h4;	// handles, instanciados aqui por 
    				// questoes de eficiencia (tempo)
};

/*
 *
z * GraphicCircle.h
 *
 *
 */

class GraphicCircle : public GraphicObj {
  public:
    //construtores
    GraphicCircle() : GraphicObj(new Circle()) {}
    GraphicCircle(Circle *c) : GraphicObj(c) {}
    
    //destrutor
    ~GraphicCircle() {}				
    
    // inherited methods redefinition
    void int_start(World *, double initial_X, double initial_Y, Int_Mode mode = LeftButton);
    void int_change(World *, double new_X, double new_Y, Int_Mode mode = LeftButton);
    void int_end(World *, double final_X, double final_Y, Int_Mode mode = LeftButton);
    
    void resize_start(World *, double X, double Y, Int_Mode mode = LeftButton);
    void resize_change(World *, double X, double Y, Int_Mode mode = LeftButton);
    void resize_end(World *, double X, double Y, Int_Mode mode = LeftButton);

    bool where_draw_string (double &x, double &y);
};

typedef Circle* Circle_ptr;
typedef GraphicCircle* GraphicCircle_ptr;

#endif ~_CIRCLE_DEFINED;
