/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti
                    Cesar N. Gon
                    Laerte F. Morgado

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

#include <values.h>
#include <math.h>
#include <assert.h>
#include "ClosestPairDC.h"
#include "Rectangle.h"
#include "Line2D.h"
#include "Circle.h"


list(ycell_ptr)   *GlobalYListPtr;
list(Point2D_ptr) *GlobalListPtr;

static int Xcmp(xcell_ptr& p1,xcell_ptr& p2);
static int Ycmp(ycell_ptr& p1,ycell_ptr& p2);


/*
 *  Animation!
 *
 */

static GraphicLine2D    *anim_linedl;  /* "d"ivide  time line->left */
static GraphicLine2D    *anim_linedr;  /* "d"ivide  time line->right */ 
static GraphicLine2D    *anim_linec;  /* "c"onquer time line */
static GraphicSegment2D *anim_segd;   /* "d"istance segment */
static GraphicPoint2D   *anim_point; 
static GraphicRectangle *anim_rect;
static GraphicCircle    *anim_circle;

#define a_linedl     ((Line2D *)anim_linedl->obj)
#define a_linedr     ((Line2D *)anim_linedr->obj)
#define a_linec      ((Line2D *)anim_linec->obj)
#define a_segd       ((Segment2D *)anim_segd->obj)
#define a_point      ((Point2D *)anim_point->obj)
#define a_rect       ((Rectangle *)anim_rect->obj)
#define a_circ       ((Circle *)anim_circle->obj)
#define anim_radius       8


static World* CPDQ_w;
static int    CPDQ_animation_depth;
static int    CPDQ_reference_depth;

static void closest_pair_dc_animation_init(World* w);
static void closest_pair_dc_animation_end();

static void closest_pair_dc_animation_init(World* w)
{
    static bool FIRST = true;
      
    if (FIRST)
      {
	  anim_linedl = new GraphicLine2D();assert(anim_linedl);
	  anim_linedr = new GraphicLine2D();assert(anim_linedr);
	  anim_linec = new GraphicLine2D();assert(anim_linec);
	  anim_segd = new GraphicSegment2D();assert(anim_segd);
	  anim_point = new GraphicPoint2D();assert(anim_point);
	  anim_rect = new GraphicRectangle();assert(anim_rect);
	  anim_circle = new GraphicCircle();assert(anim_circle);
	  FIRST = false;
      }
    anim_linedl->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
    anim_linedr->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
    anim_linec->set_all(w, FOREGROUND, BACKGROUND, 0, 1, false);
    anim_segd->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
    anim_point->set_all(w, FOREGROUND, BACKGROUND, 2, 0, false);
    anim_rect->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
    anim_circle->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false); 
}


static void closest_pair_dc_animation_end()
{
    delete anim_linedl;
    delete anim_linedr;
    delete anim_linec;
    delete  anim_segd;
    delete   anim_point;
    delete  anim_rect;
    delete   anim_circle;
}


static int Xcmp(xcell_ptr& p1,xcell_ptr& p2)
{
    Point2D_ptr  k1,k2;
    k1 = GlobalListPtr->inf(GlobalYListPtr->inf(p1->fellow)->point);
    k2 = GlobalListPtr->inf(GlobalYListPtr->inf(p2->fellow)->point);
    
    if (k1->X() < k2->X()) return -1;
    else if (k1->X() > k2->X()) return 1;
    else
      {
	  if (k1->Y() < k2->Y()) return -1;
	  else if (k1->Y() > k2->Y()) return 1;
	  else return 0;
      }
}


static int Ycmp(ycell_ptr& p1,ycell_ptr& p2)
{
    Point2D_ptr k1,k2;
    k1 = GlobalListPtr->inf(p1->point);
    k2 = GlobalListPtr->inf(p2->point);
    
    if (k1->Y() < k2->Y()) return -1;
    else if (k1->Y() > k2->Y()) return 1;
    else return 0;
}



InternalClosestPairDC::~InternalClosestPairDC()
{
    list_item E1,E2;
    
    E1 = XList.first();
    while (E1!=0)
      {
	  E2 = XList.succ(E1);
	   delete XList.del_item(E1);
	  E1 = E2;
      }
    
    
     E1 = YList.first();
    while (E1!=0)
      {
	  E2 = YList.succ(E1);
	  delete YList.del_item(E1);
	  E1 = E2;
      }
}


void InternalClosestPairDC::set(list(Point2D_ptr)&  points)
{
    list_item E;
    xcell_ptr x;
    ycell_ptr y;
    E = points.first();
    while (E!=0)
      {
	   y = new ycell; assert(y);
	   x = new xcell; assert(x);

	   
	   y->point = E;
	   y->selected = false;
	   x->fellow = YList.append(y);
	   y->fellow = XList.append(x);
	   E = points.succ(E);
       }
    GlobalYListPtr = &YList;
    GlobalListPtr  = &points;
    YList.sort(Ycmp);
    XList.sort(Xcmp);
}



Segment_ptr InternalClosestPairDC::find(list(Point2D_ptr)& points,World *w, int animation_depth, int reference_depth)
{
    double x1,x2;
    Segment_ptr result;

    CPDQ_w = w;
    CPDQ_animation_depth = animation_depth;
    CPDQ_reference_depth = reference_depth;
    
    set(points);
    if (YList.length() == 0) return 0;

    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      x1 = GlobalListPtr->inf(YList.inf(XList.inf(XList.first())->fellow)->point)->X();
      x2 = GlobalListPtr->inf(YList.inf(XList.inf(XList.last())->fellow)->point)->X();
      closest_pair_dc_animation_init(CPDQ_w);
      a_linedr->set(x2,0,x2,1);
      anim_linedr->draw(CPDQ_w,XorMode);
      a_linedl->set(x1,0,x1,1);
      anim_linedl->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)

      result = RecursiveInternalClosestPairDC(XList,YList);
    /*ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      closest_pair_dc_animation_end();
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)*/
      return result;
}



void InternalClosestPairDC::split(list(xcell_ptr)& x1,list(ycell_ptr)& y1,list(xcell_ptr)& x2,list(ycell_ptr)& y2)
{
    list_item E1,E2,Eaux,E;
    long length;
    int i;
    list(xcell_ptr) xaux;
    
    E2 = x1.last();
    length = x1.length();

    
    for (i=1;i<=length/2;i++)
      {
	  E1 = x1.pred(E2);
	  y1.inf(x1.inf(E2)->fellow)->selected = true;
	   
	  ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
	    a_circ->set(GlobalListPtr->inf(y1.inf(x1.inf(E2)->fellow)->point)->X(),GlobalListPtr->inf(y1.inf(x1.inf(E2)->fellow)->point)->Y(),anim_radius);
	    anim_circle->draw(CPDQ_w,XorMode);
	     frame_stop_mark(CPDQ_w);
	   ANIMATION_END(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
	     
	   E2 = E1;
      }
    
    Eaux = x1.succ(E2);
    
    E1 = y1.first();
    i=1;
     while (i<=length/2)
       {
	   E2 = y1.succ(E1);
	   if (y1.inf(E1)->selected == true)
	     {
		 y1.inf(E1)->selected = false;
		 
		 ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
		   a_circ->set(GlobalListPtr->inf(y1.inf(E1)->point)->X(),GlobalListPtr->inf(y1.inf(E1)->point)->Y(),anim_radius);
		   anim_circle->draw(CPDQ_w,XorMode);
		   frame_stop_mark(CPDQ_w);
		 ANIMATION_END(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
		   
		 E = y2.append(y1.del_item(E1));
		 x1.inf(y2.inf(E)->fellow)->fellow = E;
		 i++;
	     }
	   
	   E1 = E2;
       }
    x1.split(Eaux,xaux,x2);
    x1.conc(xaux);
}




Segment_ptr InternalClosestPairDC::select_min(Segment_ptr Seg1,Segment_ptr Seg2)
{
    if ((Seg1 == 0)&&(Seg2!=0))
      return Seg2;
    else if ((Seg2 == 0)&& (Seg1!=0))
      return Seg1;
    else if ((Seg1!=0)&&(Seg2!=0))
      {
	  if (Seg1->l < Seg2->l)
	    {
		ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		  a_segd->set(Seg2->s->P1(),Seg2->s->P2());
		  anim_segd->draw(CPDQ_w,XorMode);
		  frame_stop_mark(CPDQ_w);
		ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		delete Seg2->s;delete Seg2; 
		return Seg1;
	    }
	  else
	     {
		 ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		   a_segd->set(Seg1->s->P1(),Seg1->s->P2());
		   anim_segd->draw(CPDQ_w,XorMode);
		   frame_stop_mark(CPDQ_w);
		 ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		 delete Seg1->s;delete Seg1; 
		 return Seg2;
	     }
       }
    else return 0;
}






 /*
  *   This algorithm recursively finds the closest pair of a list
  * of points. It expects the list to be ordered by Y-coordinates
  * and returns the pair of points that are closest.
  *
  */

Segment_ptr InternalClosestPairDC::RecursiveInternalClosestPairDC(list(xcell_ptr)& x,list(ycell_ptr)& y)
{
    
    list(xcell_ptr) xaux;
    list(ycell_ptr) yaux,y1m,y2m;
    double median,delta = MAXDOUBLE /*infinity()*/;
    Segment_ptr Seg1=0,Seg2=0,Min1=0,Min2=0;
    double x1,x2;
    Segment_ptr result=0;

    
    if (y.length() == 1)
      {
	  delete x.del_item(x.first());
	  delete y.del_item(y.first());

	  
	  ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
	    anim_linedl->draw(CPDQ_w,XorMode);
	    anim_linedl->set_line_attr(CPDQ_w,2,0);
	    anim_linedl->draw(CPDQ_w,XorMode);
	    anim_linedr->draw(CPDQ_w,XorMode);
	    anim_linedr->set_line_attr(CPDQ_w,2,0);
	    anim_linedr->draw(CPDQ_w,XorMode);
	    frame_stop_mark(CPDQ_w);
	  ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
	     
	    return 0;
      }

    split(x,y,xaux,yaux);

    median = GlobalListPtr->inf(y.inf(x.inf(x.last())->fellow)->point)->X();
    median = ( GlobalListPtr->inf(yaux.inf(xaux.inf(xaux.first())->fellow)->point)->X() + median )/2;

    copy(y,y1m);
    copy(yaux,y2m);

    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      x1 = a_linedl->P1().X();
      x2 = a_linedr->P1().X();
      a_point->setXY(median,8);
      anim_point->draw(CPDQ_w,XorMode);
      anim_linedr->draw(CPDQ_w,XorMode);
      a_linedr->set(median,0,median,1);
      anim_linedr->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
       
    Seg1 = RecursiveInternalClosestPairDC(x,y); /* makes x = y = empty */

    
    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      anim_linedr->draw(CPDQ_w,XorMode);
      a_linedr->set(x2,0,x2,1);
      anim_linedr->set_line_attr(CPDQ_w,0,0);
      anim_linedr->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
      anim_linedl->draw(CPDQ_w,XorMode);
      a_linedl->set(median,0,median,1);
      anim_linedl->set_line_attr(CPDQ_w,0,0);
      anim_linedl->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
       
    Seg2 = RecursiveInternalClosestPairDC(xaux,yaux);
    /* makes xaux = yuax = empty */

    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      anim_linedl->draw(CPDQ_w,XorMode);
      a_linedl->set(x1,0,x1,1);
      anim_linedl->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
      a_point->setXY(median,8);
      anim_point->draw(CPDQ_w,XorMode);
      frame_stop_mark(CPDQ_w);
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)

      
    Min1 = select_min(Seg1,Seg2);
    if (Min1 != 0) delta = Min1->l; 
    delta = sqrt(delta);

    
    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      if (delta < sqrt(MAXDOUBLE))
	{
	    a_linec->set(median-delta,0,median-delta,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    a_linec->set(median,0,median,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    a_linec->set(median+delta,0,median+delta,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    frame_stop_mark(CPDQ_w);
	}
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)


      
    cut(y1m,median-delta,median);
    cut(y2m,median,median+delta);
    
    Min2 = GetMin(y1m,y2m,Min1,median,delta,delta*delta); 

    
    ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      if (delta < sqrt(MAXDOUBLE))
	{
	    a_linec->set(median-delta,0,median-delta,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    a_linec->set(median,0,median,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    a_linec->set(median+delta,0,median+delta,1);
	    anim_linec->draw(CPDQ_w,XorMode);
	    frame_stop_mark(CPDQ_w);
	}
    ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
      
    kill_y_list(y1m);
    kill_y_list(y2m);
    
    result = select_min(Min1,Min2);

    return result;
     
}



void InternalClosestPairDC::cut(list(ycell_ptr)& y,double b,double e)
{
    list_item E1,E2;
     
    E1 = y.first();

    while (E1!=0)
      {
	  E2 = y.succ(E1);
	  if ((GlobalListPtr->inf(y.inf(E1)->point)->X()<b)||(GlobalListPtr->inf(y.inf(E1)->point)->X()>e))
	    delete y.del_item(E1);
	  else
	    {
		ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
		  a_circ->set(GlobalListPtr->inf(y.inf(E1)->point)->X(),GlobalListPtr->inf(y.inf(E1)->point)->Y(),anim_radius);
		  anim_circle->draw(CPDQ_w,XorMode);
		  frame_stop_mark(CPDQ_w);
		  anim_circle->draw(CPDQ_w,XorMode);
		ANIMATION_END(CPDQ_w, CPDQ_animation_depth+5, CPDQ_reference_depth)
	      }
	  E1 = E2;  
      }
    
}



void InternalClosestPairDC::copy(list(ycell_ptr)& s,list(ycell_ptr)& t)
 {
    list_item E;
    ycell_ptr yc;

    E = s.first();

    while (E!=0)
      {
	   yc = new ycell;assert(yc);
	   yc->point = s.inf(E)->point;
	   t.append(yc);
	   E = s.succ(E);  
      }

}



void InternalClosestPairDC::kill_y_list(list(ycell_ptr)& it)
{
   list_item E1,E2;
   
   E1 = it.first();
   while (E1!=0)
      {
	   E2 = it.succ(E1);
	   delete it.del_item(E1);
	   E1 = E2;
      }
}


Segment_ptr InternalClosestPairDC::GetMin(list(ycell_ptr)& P1, list(ycell_ptr)& P2,Segment_ptr Min1,double median,double delta,double sqrdelta)
{
    list_item E1,E2;
    Segment_ptr result=0;
    double aux,aux_delta = sqrdelta;
    
    E1 = P1.first();
    E2 = P2.first();
    
    while (E1 != 0)
      {
	  Point2D p1(GlobalListPtr->inf(P1.inf(E1)->point)->X(),GlobalListPtr->inf(P1.inf(E1)->point)->Y());
	  

	   ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
	     if (delta < sqrt(MAXDOUBLE))
	       {
		   a_rect->set(median-delta,p1.Y()-delta,median+delta,p1.Y()+delta);
		   anim_rect->draw(CPDQ_w,XorMode);
	       }
	     a_circ->set(p1.X(),p1.Y(),anim_radius);
	     anim_circle->draw(CPDQ_w,XorMode);
	     frame_stop_mark(CPDQ_w);
	   ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)


	   while (( E2 != 0) && ((p1.Y() - delta) <= GlobalListPtr->inf(P2.inf(E2)->point)->Y()))
	     E2 = P2.pred(E2);
	   if (E2 == 0) E2 = P2.first();

	   while (( E2 != 0) && ((p1.Y() - delta) > GlobalListPtr->inf(P2.inf(E2)->point)->Y()))
	     E2 = P2.succ(E2);
	   if (E2 == 0) E2 = P2.last();

	   while ((E2!=0)&&((fabs(p1.Y() - GlobalListPtr->inf(P2.inf(E2)->point)->Y()))<=delta))
	     {
		 Point2D p2(GlobalListPtr->inf(P2.inf(E2)->point)->X(),GlobalListPtr->inf(P2.inf(E2)->point)->Y());
		 ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		   a_circ->set(p2.X(),p2.Y(),anim_radius);
		   anim_circle->draw(CPDQ_w,XorMode);
		   a_segd->set(p1,p2);
		   anim_segd->draw(CPDQ_w,XorMode);
		   frame_stop_mark(CPDQ_w);
		 ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)

		 aux = p1.e_distance_sqr(p2);
		 if (aux < aux_delta)
		   {
		       if (result==0)
			 {
			     result = new Segment;
			     assert(result);
			     result->s = new Segment2D(p1,p2);
			     assert(result->s);
			     result->l = aux;
			   }
		       else
			 {
			     ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
			       a_segd->set(result->s->P1(),result->s->P2());
			       anim_segd->draw(CPDQ_w,XorMode);
			       frame_stop_mark(CPDQ_w);
			     ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
			     result->s->set(p1,p2);
			     result->l = aux;
			 }
		       aux_delta = aux;
		   }
		 else
		   {
		       ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
			 a_segd->set(p1,p2);
			 anim_segd->draw(CPDQ_w,XorMode);
			 frame_stop_mark(CPDQ_w);
		       ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		   }
		 ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		   a_circ->set(p2.X(),p2.Y(),anim_radius);
		   anim_circle->draw(CPDQ_w,XorMode);
		   frame_stop_mark(CPDQ_w);
		 ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
		 E2 = P2.succ(E2);
	     }
	   ANIMATION_BEGIN(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
	     a_circ->set(p1.X(),p1.Y(),anim_radius);
	     anim_circle->draw(CPDQ_w,XorMode);
	     if (delta < sqrt(MAXDOUBLE))
	       {
		   a_rect->set(median-delta,GlobalListPtr->inf(P1.inf(E1)->point)->Y() - delta,median+delta,GlobalListPtr->inf(P1.inf(E1)->point)->Y() + delta);
		   anim_rect->draw(CPDQ_w,XorMode);
	       }
	     frame_stop_mark(CPDQ_w);
	   ANIMATION_END(CPDQ_w, CPDQ_animation_depth, CPDQ_reference_depth)
	   E1 = P1.succ(E1);
      };

      return result;
}






