/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * ConvexHull.h - definicao da superclasse de algoritmos para a construcao de
 *                envoltorias convexas para conjuntos de pontos. Implementacao
 *                dos algoritmos Jarvis' March e Graham's Scan (e aproveitamento
 *		  destes para a construcao de cascas convexas em tempo NAO otimo).
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _CONVEX_HULL_DEFINED
#define _CONVEX_HULL_DEFINED
 
#include "External.h"		// obrigatorio a todo modulo externo
#include "Algorithm.h"
#include "Point2D.h"
#include "Polygon.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list;

#ifndef _Polygon_ptr_list
#define _Polygon_ptr_list
declare(list,Polygon_ptr)
#endif ~_Polygon_ptr_list

/*
 * Convex Hull of polygons
 *
 */
 
list(Point2D_ptr) ch_generic_polygon(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);
list(Point2D_ptr) ch_simple_polygon(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);
list(Point2D_ptr) ch_starshaped_polygon(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);

/*
 * Convex Hull for point sets
 *
 */
 
list(Point2D_ptr) jarvis_march(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);
list(Point2D_ptr) graham_scan(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);
list(Polygon_ptr) convex_layers_jarvis(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);
list(Polygon_ptr) convex_layers_graham(list(Point2D_ptr) &points, World *w = 0, int animation_depth = 0, int reference_depth = 0);

// Modelagem dos algoritmos acima em termos do ambiente GeoLab 

class ConvexHull : public Algorithm {
  public:
    bool animation() { return true; }

    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
};

class JarvisMarch : public ConvexHull {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};

class GrahamScan : public ConvexHull {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};

class ConvexLayers : public ConvexHull {
};

class ConvexLayersJarvis : public ConvexLayers {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};

class ConvexLayersGraham : public ConvexLayers {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};

/*
 * Polygon Hulls
 *
 */
 
class PolygonHull : public ConvexHull {
};

class GenericPolygonHull : public PolygonHull {
    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};

class SimplePolygonHull : public GenericPolygonHull {
    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};
 
class StarShapedPolygonHull : public SimplePolygonHull {
    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};
 
Algorithm* jarvis_march_link();
Algorithm* graham_scan_link();
Algorithm* convex_layers_jarvis_link();
Algorithm* convex_layers_graham_link();

Algorithm* generic_polygon_hull_link();
Algorithm* simple_polygon_hull_link();
Algorithm* starshaped_polygon_hull_link();

#endif ~_CONVEX_HULL_DEFINED;





