/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Coords2D.h - contem a definicao das classes PlaneCoords que representa
 *              coordenadas planares de um ponto atraves de coordenadas
 *              homogeneas, e da classe PlaneMatrix, que define a matriz
 *              de transformacoes sobre as coordenadas (translacao, rotacao).
 *  
 *              link flag (-lm)
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    31.10.91  13:45	Welson R. Jacometti	Arquivo Criado
 *
 */

#ifndef _COORDS2D_DEFINED
#define _COORDS2D_DEFINED

#include <math.h>
#include <iostream.h>
#include <stdio.h>
#include "global.h"
#include "tools.h"

/*
 * Constantes exportaveis
 *
 */

#define Plane_XCoord   0
#define Plane_YCoord   1
#define Plane_WCoord   2

class PlaneCoords {
public:
  PlaneCoords() {}
  PlaneCoords(double x, double y, double w) {
      coords[Plane_XCoord] = x;
      coords[Plane_YCoord] = y;
      coords[Plane_WCoord] = w;      
  }

  void normalize() {
      if (!at_infinity())
	if (Positive(coords[Plane_WCoord]))
	  set(coords[Plane_XCoord]/coords[Plane_WCoord],
	      coords[Plane_YCoord]/coords[Plane_WCoord],
	      1);
	else
	  set(-coords[Plane_XCoord]/coords[Plane_WCoord],
	      -coords[Plane_YCoord]/coords[Plane_WCoord],
	      -1);	  
  }

  double X() const {
      if (!Zero(coords[Plane_WCoord]))
        return coords[Plane_XCoord] / coords[Plane_WCoord];
      else
        return coords[Plane_XCoord];
  }
  
  double Y() const {
      if (!Zero(coords[Plane_WCoord]))
        return coords[Plane_YCoord] / coords[Plane_WCoord];
      else
        return coords[Plane_YCoord];
  }
  
  void setXY(double x, double y) {
      coords[Plane_XCoord] = x;
      coords[Plane_YCoord] = y;
      coords[Plane_WCoord] = 1;
  }

  void set(double x, double y, double w) {
      coords[Plane_XCoord] = x;
      coords[Plane_YCoord] = y;
      coords[Plane_WCoord] = w;
  }
  
  void set_at_infinity(double x, double y) {
      coords[Plane_XCoord] = x;
      coords[Plane_YCoord] = y;
      coords[Plane_WCoord] = 0;
  }

  bool at_infinity() const { return Zero(coords[Plane_WCoord]); }
    
  double& operator [] (int index) { return coords[index]; }
  double operator * (PlaneCoords) const;
  void print();

  void load(FILE *f);
  void save(FILE *f);
            
private:
  double coords[3];
};

class PlaneMatrix {
public:
  PlaneMatrix() {}

  double* operator [] (int index) const { return (double *)(matrix[index]); }
  PlaneMatrix operator * (const PlaneMatrix&);
  void ident();
  void trans(double tx, double ty);
  void rotat(double angle);
  void print();
        
private:
  void maketrans(double tx, double ty);
  void makerotat(double angle);
  double matrix[3][3];
};

// tools

PlaneCoords cross(PlaneCoords&, PlaneCoords&);
PlaneCoords multmatrix(PlaneCoords&, PlaneMatrix&);

#endif ~_COORDS2D_DEFINED;

