/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Message.h
 *
 * Nota: O mapeamento de objetos de natureza nao geometrica na hierarquia de
 *       objetos geometricos visa estabelecer uma forma homogenea de integra-
 *       cao entre os algoritmos geometricos e a interface do ambiente, alem
 *       de propiciar a inclusao de algoritmos geometricos que produzem valo-
 *       res simples, como booleanos (left_turn, right_turn...), inteiros, 
 *       reais e, eventualmente, strings. Neste ultimo caso, as aplicacoes
 *       sao muito variadas. Via de regra, um algoritmo geometrico recebe
 *       como entrada um objeto geometrico e sua saida e' tambem um objeto
 *       geometrico. O que acontece se o algoritmo quiser sinalizar uma situ-
 *       acao de erro ? Neste caso, o "objeto geometrico" produzido e' uma
 *       mensagem, que, devido `a forma de implementacao, possui metodos que
 *       possibilitam que seja mostrada e entao descartada.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _MESSAGE_DEFINED
#define _MESSAGE_DEFINED

#include <stdio.h>
#include <iostream.h>
#include "global.h"
#include "World.h"
#include "Obj.h"
#include "String.h"
#include "ExternalObj_id.h"	// obrigatorio em todo objeto externo

// forward declaration
class GraphicMessage;
 
class Message : public Obj {
  public:
    // construtores
    Message(char *s = "Message", ObjType t = Generic_t, bool v = false) : Obj(s, t, v) { set("No message"); }
    Message( Message &s) : Obj(s.what_is(), s.stype, s.volati) { set(s.get()); }
    
    //destrutor
    ~Message() {}

    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                  Message --> GraphicMessage
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Message(*this);
    }
        
    // query & set functions
    void set(String s) { message = s; }
    String get()  { return message; }
    
    // inherited methods redefinition
    ObjType type()  { return Message_t; }
    
    void print()  { cout << "Message: " << message.value() << endl; }

    void draw(World *w, GC gc = 0) ;

  protected:
    String message;
};

/*
 *
 * GraphicMessage
 *
 *
 */

class GraphicMessage : public GraphicObj {
  public:
    // construtores
    GraphicMessage() : GraphicObj(new Message()) {}
    GraphicMessage(Message *m) : GraphicObj(m) {}
    
    //destrutor
    ~GraphicMessage() {}
    
    // inherited methods redefinition
    void draw(World *, DrawMode);
};

typedef Message* Message_ptr;
typedef GraphicMessage* GraphicMessage_ptr;

#endif ~_MESSAGE_DEFINED;
