/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * MinSpanningTree.C - implementacao do algoritmo para construcao da Arvore
 *                     Euclidiana Espalhada Minima. Algoritmo retirado
 *                     da implementacao do LEDA para efeito de demonstracao.
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "MinSpanningTree.h"
#include "List.h"
#include "QuadEdge.h"
#include "voronoi.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare (list,Point2D_ptr)
#endif ~_Point2D_ptr_list;

// O campo booleano de informacoes para aresta e' utilizado para decidir
// quais arestas da triangulacao de Delaunay devem ser descartadas apos
// a execucao do algoritmo.

#ifndef _Point2D_ptrbool_UGRAPH
#define _Point2D_ptrbool_UGRAPH
declare2 (UGRAPH,Point2D_ptr,bool)
#endif ~_Point2D_ptrbool_UGRAPH;

/*
 **** includes e variaveis para animacao ***/
 
static GraphicSegment2D *anim_s;

#define anim_segment	((Segment2D *)anim_s->obj)

void spanning_animation_init(World *w) {
    static bool FIRST = true;
    
    if (FIRST) {
      anim_s = new GraphicSegment2D();
      FIRST = false;
    }
    anim_s->set_all(w, FOREGROUND, BACKGROUND, 4, 0, false);
}
 
/*
 *******************************************/
  
static edge_array(real)* C;

int cmp_edges(edge &e1, edge &e2) {
    return (*C)[e1] - (*C)[e2];
}

SpanningTree *min_spanning_tree(UGRAPH(Point2D_ptr,bool) &G, edge_array(real) &cost, World *w, int animation_depth, int reference_depth) {
    node nv, nw;
    edge e;
    node_partition Q(G);

    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      spanning_animation_init(w);
    ANIMATION_END(w, animation_depth, reference_depth)
        
    list(edge) OEL = G.all_edges();
    C = &cost;
    OEL.sort(cmp_edges);

    forall (e, OEL) {
      nv = G.source(e);
      nw = G.target(e);
      
      ANIMATION_BEGIN(w, animation_depth, reference_depth)
      	node i = G.source(e);
      	node f = G.target(e);
        anim_segment->set(*(Point2D_ptr)G.inf(i), *(Point2D_ptr)G.inf(f));
        for (int count = 0; count < 10; count++) {
          anim_s->draw(w, XorMode);
          frame_delay_mark(w);
        }
      ANIMATION_END(w, animation_depth, reference_depth)
      
      if (!(Q.same_block(nv, nw))) {
      	Q.union_blocks(nv, nw);
      	G.assign(e, false);
      	
      	ANIMATION_BEGIN(w, animation_depth, reference_depth)
      	  node i = G.source(e);
          node f = G.target(e);
          anim_segment->set(*(Point2D_ptr)G.inf(i), *(Point2D_ptr)G.inf(f));
          anim_s->draw(w, CopyMode);
          frame_stop_mark(w);      	
      	ANIMATION_END(w, animation_depth, reference_depth)
      	
      }
    }

    // remove arestas da triangulacao
    
    forall (e, OEL) {
      if (G.inf(e))
        G.del_edge(e);
    }
    
    // G contem apenas a arvore Euclidiana espalhada minima

    SpanningTree *res = new SpanningTree();
    res->set_graph(G);
    res->subtype(MinimumSpanningTree_t);

    return res;
}


// mapeamento

list(Point2D_ptr) extract_points(const SelectedObjects &SO, const GraphicObjects &GO) {
    list(Point2D_ptr) res;
    list_item it;
    forall(it, SO) {                          // is_able_to_handle deve ser
      res.push((Point2D *)GO[it]->obj);       // chamado antes para certificar
    }                                         // que sao todos pontos
    return res;
}

// traducao de QuadEdge em Graph

static UGRAPH(Point2D_ptr,bool) *rec_graph;
static node rec_node;

QuadEdge_forall_bf_BEGIN(recursive_introduce_nodes,e)
    if (!already) {
      rec_node = rec_graph->new_node();
      rec_graph->assign(rec_node, (Point2D_ptr)eaux->org()->reproduct());
      already = true;
    }
    eaux->info((void *)rec_node);
QuadEdge_forall_bf_END(recursive_introduce_nodes,e)

QuadEdge_forall_df_BEGIN(recursive_introduce_edges,e)
    rec_graph->new_edge((node)e->info(), (node)e->sym()->info(), true);
QuadEdge_forall_df_END(recursive_introduce_edges,e)

void translate_QuadEdge_Graph(QuadEdge_ptr q, UGRAPH(Point2D_ptr,bool) &g) {
    QEdge_ptr e = q->root();
    if (!e) return;
    g.clear(); rec_graph = &g;
    rec_flag = *(e->oflag());
    recursive_introduce_nodes(e);
    rec_flag = *(e->flag());
    recursive_introduce_edges(e);
}
	
bool EMST::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    if (SO.size() == 1 && GO[SO.head()]->type() == QuadEdge_t && GO[SO.head()]->subtype() == DelaunayTriangulation_t)
      return true;
    else {
      list_item it;
      forall(it, SO) {
        if (GO[it]->type() != Point2D_t) {
      	  SO.init_iterator();
      	  return false;
      	}    
      }
    }
    
    return true;
}

ResultObjects EMST::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    
    if (!SO.empty()) {
      QuadEdge_ptr triang; 	
      bool created = false;
      	
      if (GO[SO.head()]->type() != QuadEdge_t) {
      	list(Point2D_ptr) points = extract_points(SO, GO);
      	triang = delaunay_triangulation(points, w, animation_depth+1, reference_depth);
      	created = true;
      }
      else
      	triang = ((QuadEdge *)GO[SO.head()]->obj);  // extrai triangulacao

      UGRAPH(Point2D_ptr,bool) graph;
      translate_QuadEdge_Graph(triang, graph);

      edge_array(real) cost(graph);
      edge x;
      forall_edges(x, graph) {
      	node i = graph.source(x);
      	node f = graph.target(x);
      	cost[x] = ((Point2D_ptr)graph.inf(i))->e_distance(*(Point2D_ptr)graph.inf(f));
      }
      
      SpanningTree_ptr st = min_spanning_tree(graph, cost, w, animation_depth, reference_depth);

      if (st)
        result.push(st);
      
      if (created)
        delete triang;
    }
    
    return result;
}
   
Obj* EMST::instantiation(ObjType) {
    return 0; 	// falta implementar (tambem armazenamento para SpanningTree)
}

Algorithm* emst_link() {
    return new EMST();
}




