/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Point2D.C - Point2D & GraphicPoint2D methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    19.11.91  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Obj.h"
#include "Point2D.h"

GraphicObj* Point2D::new_GraphicObj() {
    return new GraphicPoint2D(this);
}

bool Point2D::circular_less( Point2D &q,  Point2D &ref_point)  {
    return circular_comp_ref(*this, q, ref_point) == Less; 
}
    
bool Point2D::circular_equal( Point2D &q,  Point2D &ref_point)  {
    return circular_comp_ref(*this, q, ref_point) == Equal;
}
   
inCircleType Point2D::inCircle( Point2D &a,  Point2D &b,  Point2D &c)  {
    double xaya, xbyb, xcyc, xxyy;
    double d1, d2, d3, d4, val;
    CompType res;
    
    xaya = (a.X() * a.X()) + (a.Y() * a.Y());
    xbyb = (b.X() * b.X()) + (b.Y() * b.Y());
    xcyc = (c.X() * c.X()) + (c.Y() * c.Y());

    xxyy = (X() * X()) + (Y() * Y());
    
    d1 = (b.Y() * xcyc) + (c.Y() * xxyy) + (Y() * xbyb) -
         (b.Y() * xxyy) - (c.Y() * xbyb) - (Y() * xcyc);
         
    d2 = (b.X() * xcyc) + (c.X() * xxyy) + (X() * xbyb) -
         (b.X() * xxyy) - (c.X() * xbyb) - (X() * xcyc);
         
    d3 = (b.X() * c.Y()) + (c.X() * Y()) + (X() * b.Y()) -
         (b.Y() * c.X()) - (c.Y() * X()) - (Y() * b.X());
         
    d4 = (b.X() * c.Y() * xxyy) + (c.X() * Y() * xbyb) + (X() * b.Y() * xcyc) -
         (b.Y() * c.X() * xxyy) - (c.Y() * X() * xbyb) - (Y() * b.X() * xcyc);
         
    val = (a.X() * d1) - (a.Y() * d2) + (xaya * d3) - d4;
    
    res = comp (0, val);

    if (res == Less)
      return Inside;
    else
      if (res == Equal)
        return Boundary;
      else
        return Outside;    
}

double Point2D::e_distance( Point2D &q)  {
    return sqrt(e_distance_sqr(q));
}
    
double Point2D::e_distance_sqr( Point2D &q)  {
    double sqr1, sqr2;
    
    sqr1 = (X() - q.X()) * (X() - q.X());
    sqr2 = (Y() - q.Y()) * (Y() - q.Y());
    
    return sqr1 + sqr2;
}    

double Point2D::angle( Point2D &q)  {
    double delta_x, delta_y;
    double ang;
    
    delta_x = q.X() - X();
    delta_y = q.Y() - Y();
    
    if (zero(delta_x) && zero(delta_y))
      return 2 * PI;

    if (zero(delta_x))
      if (less(delta_y, 0))
        return (3 * PI) / 2;
      else
        return PI / 2;

    if (zero(delta_y))
      if (less(delta_x, 0))
        return PI;
      else
        return 0;
      
    ang = atan(delta_y / delta_x);
    
    if (less(delta_x, 0))			// segundo e terceiro quadrantes
      ang += PI;
    else
      if (less(0, delta_x) && less(delta_y, 0)) // quarto quadrante
        ang += 2 * PI;

    return ang;
}

// inherited methods redefinition

void Point2D::unfill_handles(World *w, bool show_handle)  {
    if (!at_infinity() && show_handle)
      unfill_handle(w, X(), Y());
}

void Point2D::unfill_circular_handles(World *w, bool show_handle)  {
    if (!at_infinity() && show_handle)
      unfill_circular_handle(w, X(), Y());
}

void Point2D::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (!at_infinity())
      if (show_handle)
        if (highlight)
          fill_handle(w, gc, X(), Y());
        else
          draw_handle(w, gc, X(), Y());
      else {
        int cx = w->xcoord(X());
        int cy = w->ycoord(Y());

        w->DrawLine(gc, w->virtual_xcoord(cx - 2), w->virtual_ycoord(cy), 
                        w->virtual_xcoord(cx + 2), w->virtual_ycoord(cy));
        w->DrawLine(gc, w->virtual_xcoord(cx), w->virtual_ycoord(cy - 2), 
                        w->virtual_xcoord(cx), w->virtual_ycoord(cy + 2));
      }
}

void Point2D::draw_circular_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (!at_infinity())
      if (show_handle)
        if (highlight)
          fill_circular_handle(w, gc, X(), Y());
        else
          draw_circular_handle(w, gc, X(), Y());
      else {
        int cx = w->xcoord(X());
        int cy = w->ycoord(Y());

        w->DrawLine(gc, w->virtual_xcoord(cx - 2), w->virtual_ycoord(cy), 
                        w->virtual_xcoord(cx + 2), w->virtual_ycoord(cy));
        w->DrawLine(gc, w->virtual_xcoord(cx), w->virtual_ycoord(cy - 2), 
                        w->virtual_xcoord(cx), w->virtual_ycoord(cy + 2));
      }
}

void Point2D::draw(World *w, GC gc)  {
    if (!at_infinity())
      w->DrawPoint(gc, X(), Y());
}

/*
 *
 * GraphicPoint2D methods
 *
 *
 */

// cast macro

#define point	((Point2D *)obj)

// inherited methods redefinition

void GraphicPoint2D::int_start(World *, double initial_X, double initial_Y, Int_Mode) {
    point->setXY(initial_X, initial_Y);
}

void GraphicPoint2D::int_change(World *, double new_X, double new_Y, Int_Mode) {
    point->setXY(new_X, new_Y);
}

void GraphicPoint2D::int_end(World *, double final_X, double final_Y, Int_Mode) {
    point->setXY(final_X, final_Y);
}

void GraphicPoint2D::resize_start(World *, double X, double Y, Int_Mode) {}

void GraphicPoint2D::resize_change(World *, double X, double Y, Int_Mode) {
    point->setXY(X, Y);
}

void GraphicPoint2D::resize_end(World *, double X, double Y, Int_Mode) {
    point->setXY(X, Y);
}

void GraphicPoint2D::draw(World *w, DrawMode dm) {
    if (saved_dm != dm)
      set_draw_mode(w, dm);
    saved_dm = dm;
    ((Point2D *)obj)->draw_circular_handles(w, gch, is_selected, show_handle);
    if (label.length()) {
      double x,y;
      if (where_draw_string (x, y))
	w->DrawCenterString (gch, x, y, label.value(), label.length());
    }
}
     
bool GraphicPoint2D::where_draw_string (double &x, double &y) {    
    x = ((Point2D *)obj)->X();
    y = ((Point2D *)obj)->Y()-14;
    return true;
}

// related functions

Point2D gravitycentre( Point2D &a,  Point2D &b,  Point2D &c)  {
    Point2D res;
    
    res.setXY((a.X() + b.X() + c.X()) / 3, (a.Y() + b.Y() + c.Y()) / 3);

    return res;
}

Point2D incentre( Point2D &a,  Point2D &b,  Point2D &c, double &radius)  {
    Point2D res;
    double rab, rbc, rca, t, s;
    
    rab = a.e_distance(b);
    rbc = b.e_distance(c);
    rca = c.e_distance(a);
    
    t = rab + rbc + rca;
    s = t/2;

    if (t != 0) {    
      res.setXY(((rab * c.X()) + (rbc * a.X()) + (rca * b.X())) / t,
                ((rab * c.Y()) + (rbc * a.Y()) + (rca * b.Y())) / t);
      radius = sqrt(((s - rab) * (s - rbc) * (s - rca)) / s);
    }
    
    return res;
}

Point2D circumcentre( Point2D &a,  Point2D &b,  Point2D &c)  {
    Point2D res;
    
    if (!a.colinear(b, c)) {
      double xca, yca, xba, yba;
      double det1, det2, det3;
      
      xca = c.X() - a.X();
      yca = c.Y() - a.Y();
      xba = b.X() - a.X();
      yba = b.Y() - a.Y();
      
      det1 = (((xca * xca) + (yca * yca)) * yba) - (((xba * xba) + (yba * yba)) * yca);
      det2 = (((xba * xba) + (yba * yba)) * xca) - (((xca * xca) + (yca * yca)) * xba);
      det3 = 2 * ((xca * yba) - (xba * yca));
      
      res.setXY(a.X() + (det1 / det3), a.Y() + (det2 / det3));
    }
    
    return res;
}

Point2D baricentre( Point2D &a,  Point2D &b,  Point2D& c)  {
    Point2D res;
    
    if (!a.colinear(b, c))
      res.setXY((a.X() + b.X() + c.X()) / 3, a.Y() + b.Y() + c.Y() / 3);
      
    return res;
}

Point2D middle( Point2D &p,  Point2D &q)  {
    Point2D res;
    
    res.setXY((p.X() + q.X()) / 2, (p.Y() + q.Y()) / 2);

    return res;
}

Point2D get_max_X( list(Point2D_ptr) &Points)  {
    Point2D_ptr p_ptr, max = Points.head();
    forall (p_ptr, Points) {
      if (lexico_Xcomp(*p_ptr, *max) == Greater) 
        max=p_ptr;
    }
    return *max;
}

Point2D get_max_Y( list(Point2D_ptr) &Points)  {
    Point2D_ptr p_ptr, max = Points.head();
    forall (p_ptr, Points) {
      if (lexico_Ycomp(*p_ptr, *max) == Greater) 
        max=p_ptr;
    }
    return *max;
}

Point2D get_min_X( list(Point2D_ptr) &Points)  {
    Point2D_ptr p_ptr, min = Points.head();
    forall (p_ptr, Points) {
      if (lexico_Xcomp(*p_ptr, *min) == Less) 
        min=p_ptr;
    }
    return *min;
}

Point2D get_min_Y( list(Point2D_ptr) &Points)  {
    Point2D_ptr p_ptr, min = Points.head();
    forall (p_ptr, Points) {
      if (lexico_Ycomp(*p_ptr, *min) == Less) 
        min=p_ptr;
    }
    return *min;
}

list_item it_get_max_X( list(Point2D_ptr) &Points)  {
    list_item it, max = Points.first();
    forall_list_items(it, Points) {
      if (lexico_Xcomp(*Points[it], *Points[max]) == Greater) 
        max=it;
    }
    return max;
}

list_item it_get_max_Y( list(Point2D_ptr) &Points)  {
    list_item it, max = Points.first();
    forall_list_items(it, Points) {
      if (lexico_Ycomp(*Points[it], *Points[max]) == Greater) 
        max=it;
    }
    return max;
}

list_item it_get_min_X( list(Point2D_ptr) &Points)  {
    list_item it, min = Points.first();
    forall_list_items(it, Points) {
      if (lexico_Xcomp(*Points[it], *Points[min]) == Less) 
        min=it;
    }
    return min;
}

list_item it_get_min_Y( list(Point2D_ptr) &Points)  {
    list_item it, min = Points.first();
    forall_list_items(it, Points) {
      if (lexico_Ycomp(*Points[it], *Points[min]) == Less) 
        min=it;
    }
    return min;
}

Point2D lower_angle_cw( list(Point2D_ptr) &Points,  Point2D &ref)  {
    Point2D res;
					// busca linear
    if (Points.empty())
      res.set_at_infinity(0, 0);
    else {
      res = *Points.head();
      if (Points.size() > 1) {
        list_item i = Points.first(), inext = Points.succ(i);
        SideType s;
        do {
          s = Points[inext]->which_side(ref, *Points[i]);
      	  if (s == Left || (s == Colinear && ref.e_distance_sqr(*Points[inext]) > ref.e_distance_sqr(*Points[i])))
      	    i = inext;
      	  inext = Points.succ(inext);
      	} while(inext);
      	res = *Points[i];
      }
    }
    
    return res;
}

Point2D lower_angle_ccw( list(Point2D_ptr) &Points,  Point2D &ref)  {
    Point2D res;
					// busca linear
    if (Points.empty())
      res.set_at_infinity(0, 0);
    else {
      res = *Points.head();
      if (Points.size() > 1) {
        list_item i = Points.first(), inext = Points.succ(i);
        SideType s;
        do {
          s = Points[inext]->which_side(ref, *Points[i]);
      	  if (s == Right || (s == Colinear && ref.e_distance_sqr(*Points[inext]) > ref.e_distance_sqr(*Points[i])))
      	    i = inext;
      	  inext = Points.succ(inext);
      	} while(inext);
      	res = *Points[i];
      }
    }
    
    return res;
}

list_item it_lower_angle_cw( list(Point2D_ptr) &Points,  Point2D &ref)  {
    list_item i, inext;
					// busca linear
    if (Points.empty())
      i = 0;
    else {
      i = Points.first();
      if (Points.size() > 1) {
        inext = Points.succ(i);
        SideType s;
        do {
          s = Points[inext]->which_side(ref, *Points[i]);
    	  if (s == Left || (s == Colinear && ref.e_distance_sqr(*Points[inext]) > ref.e_distance_sqr(*Points[i])))
      	    i = inext;
            inext = Points.succ(inext);
        } while(inext);
      }
    }
    
    return i;
}

list_item it_lower_angle_ccw( list(Point2D_ptr) &Points,  Point2D &ref)  {
    list_item i, inext;
					// busca linear
    if (Points.empty())
      i = 0;
    else {
      i = Points.first();
      if (Points.size() > 1) {
        inext = Points.succ(i);
        SideType s;
        do {
          s = Points[inext]->which_side(ref, *Points[i]);
    	  if (s == Right || (s == Colinear && ref.e_distance_sqr(*Points[inext]) > ref.e_distance_sqr(*Points[i])))
      	    i = inext;
            inext = Points.succ(inext);
        } while(inext);
      }
    }
    
    return i;
}

void remove_duplicated_points(list(Point2D_ptr) &Points) {
    if (!Points.empty()) {
        list_item it = Points.first();
        list_item ni = Points.succ(it);
        while (it && ni) {
          if (*Points[it] == *Points[ni])
            Points.del_item(ni);
          else
            it = ni;
          ni = Points.succ(it);
        }
    }
}

void remove_colinear_points(list(Point2D_ptr) &Points) {
    list_item it = Points.first();
    while (Points.size() > 2 && Points.cyclic_succ(it) != Points.first()) {
      if (Points[it]->colinear(*Points[Points.succ(it)], *Points[Points.cyclic_succ(Points.succ(it))]))
        Points.del_item(Points.succ(it));
      else
        it = Points.succ(it);
    }
}

CompType lexico_Xcomp( Point2D &p1,  Point2D &p2)  {
    CompType res = comp(p1.X(), p2.X());
    if (res == Equal)
      return comp(p1.Y(), p2.Y());

    return res;
}

CompType lexico_Ycomp( Point2D &p1,  Point2D &p2)  {
    CompType res = comp(p1.Y(), p2.Y());
    if (res == Equal)
      return comp(p1.X(), p2.X());

    return res;
}

CompType lexico_Xcomp_ptr( Point2D_ptr &p1,  Point2D_ptr &p2)  {
    return lexico_Xcomp(*p1, *p2);
}

CompType lexico_Ycomp_ptr( Point2D_ptr &p1,  Point2D_ptr &p2)  {
    return lexico_Ycomp(*p1, *p2);
}

static Point2D cr;
void set_circular_comp_reference( Point2D &q)  { 
   cr = q;	// problema antigo corrigido com operador "=" (wrj)
}

CompType circular_comp( Point2D &p1,  Point2D &p2)  {
    return circular_comp_ref(p1, p2, cr);
}

CompType circular_comp_ref( Point2D &p1,  Point2D &p2,  Point2D &ref)  {
    if (ref.colinear(p1, p2)) {
      if (ref == p1)
        return Less;
      else
        if (ref == p2)
          return Greater;
        else
          if (p1 == p2)
            return Equal;
            
      return comp(ref.e_distance_sqr(p1), ref.e_distance_sqr(p2));
    }
    
    if (ref.left_turn(p1, p2))
      return Less;
    else
      return Greater;      
}

CompType circular_comp_ptr( Point2D_ptr &p1,  Point2D_ptr &p2)  {
    return circular_comp_ref(*p1, *p2, cr);
}

CompType circular_comp_ptr_ref( Point2D_ptr &p1,  Point2D_ptr &p2,  Point2D_ptr &ref)  {
    return circular_comp_ref(*p1, *p2, *ref);
}

