/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * Polygon_test.C
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.01.92  13:40 	Cesar N. Gon 	  	Arquivo criado
 *
 */

#include "Polygon_test.h"
#include "Boolean.h"

list(Point2D_ptr) get_polygon_points(Polygon_ptr P) {
    return P->copy_vertices();
}

list(Segment2D_ptr) get_polygon_segments(Polygon_ptr P) {
    return P->copy_edges();
}

/*
 * implementacao dos metodos
 *
 */
 
bool PolygonTools::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    list_item it;
    
    forall(it, SO) {
      if (GO[it]->type() != Polygon_t) {
        SO.init_iterator();
        return false;
      }
    }

    return true;
}

ResultObjects get_points::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Point2D_ptr) Points;
    list_item it;
    Obj_ptr obj;
    Point2D_ptr p;

    if (!SO.empty())
      forall (it, SO)  {
        obj=GO[it]->obj;
	if (obj->type()==Polygon_t) {
	  Points=get_polygon_points((Polygon_ptr)obj);
	  forall (p,Points) {
	    Result.push(p);
	  }
	}
      }
      
    return Result;
}

ResultObjects get_segments::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Segment2D_ptr) Segments;
    list_item it;
    Obj_ptr obj;
    Segment2D_ptr s;
    
    if (!SO.empty())
      forall (it, SO)  {
        obj=GO[it]->obj;
	if (obj->type()==Polygon_t) {
	  Segments=get_polygon_segments((Polygon_ptr)obj);
	  forall (s,Segments) {
            Result.push(s);
          }
	}
      }
      
    return Result;
}


ResultObjects test_and_set_simple::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list_item it;
    Obj_ptr obj;
    bool b, ans = true, ok = false;
    Boolean *Bool_ptr;
    
    if (!SO.empty()) {
	forall (it, SO)  {
	    obj=GO[it]->obj;
	    if (obj->type()==Polygon_t) {
		ok = true;
		if (b = ((Polygon_ptr)obj)->is_simple()) 
		  (Polygon_ptr)obj->subtype(Simple_t);
		ans = ans && b;
	      }
	}
	if (ok) {
	    Bool_ptr = new Boolean (ans);
	    Bool_ptr->obj_volatile(true);
	    Result.push(Bool_ptr);
	}
    }
    return Result;
}

ResultObjects test_and_set_convex::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list_item it;
    Obj_ptr obj;
    bool b, ans = true, ok = false;
    Boolean *Bool_ptr;
    
    if (!SO.empty()) {
	forall (it, SO)  {
	    obj=GO[it]->obj;
	    if (obj->type()==Polygon_t) {
		ok = true;
		if (b = ((Polygon_ptr)obj)->is_convex())
		  (Polygon_ptr)obj->subtype(Convex_t);
		ans = ans && b;
	    }
	}
	if (ok) {
	    Bool_ptr = new Boolean (ans);
	    Bool_ptr->obj_volatile(true);
	    Result.push(Bool_ptr);
	}
    }
    return Result;
}

Algorithm* get_points_link() {
    return new get_points();
}

Algorithm* get_segments_link() {
    return new get_segments();
}

Algorithm* test_and_set_simple_link() {
    return new test_and_set_simple();
}

Algorithm* test_and_set_convex_link() {
    return new test_and_set_convex();
}


