/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * Proximity.C - mapeamento dos algoritmos para problemas de proximidade
 *               implementados em proximity.C (.h)
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Proximity.h"
#include "basic.h"
#include "List.h"
#include "Segment2D.h"
#include "proximity.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list

#ifndef _Segment2D_ptr_list
#define _Segment2D_ptr_list
declare(list,Segment2D_ptr)
#endif ~_Segment2D_ptr_list

static list(Point2D_ptr) extract_points(const SelectedObjects &SO, const GraphicObjects &GO) {
    list(Point2D_ptr) res;
    list_item it;
    forall(it, SO) {			 	// is_able_to_handle deve ser
      res.push((Point2D *)GO[it]->obj);		// chamado antes para certificar
    }						// que sao todos pontos
    return res;
}

bool ProxPoints::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    list_item it;
    
    forall(it, SO) {
      if (GO[it]->type() != Point2D_t) {
      	SO.init_iterator();
      	return false;
      }
    }
    
    return true;
}

ResultObjects ClosestPairBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Segment2D_ptr cpair;
    
    if (!SO.empty()) {
      if ((cpair = closest_pair_bf(extract_points(SO, GO), w, animation_depth, reference_depth)) != 0)
        result.push(cpair);
    }

    return result;
}
  
ResultObjects AllNearestNeighborsBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Segment2D_ptr) all_nn;
    Segment2D_ptr s;

    if (!SO.empty()) {
      all_nn = all_nearest_neighbors_bf(extract_points(SO, GO), w, animation_depth, reference_depth);
      forall(s, all_nn) {
        result.push(s);
      }
    }
      
    return result;
}


ResultObjects ClosestPairDC::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Segment2D_ptr cpair;

    if (!SO.empty()) {
      if ((cpair = closest_pair_dc(extract_points(SO, GO), w, animation_depth, reference_depth)) != 0)
        result.push(cpair);
    }

    return result;
}
  
ResultObjects AllNearestNeighborsDC::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Segment2D_ptr) all_nn;
    Segment2D_ptr s;

    if (!SO.empty()) {
      all_nn = all_nearest_neighbors_dc(extract_points(SO, GO), w, animation_depth, reference_depth);
      forall(s, all_nn) {
        result.push(s);
      }
    }
      
    return result;
}


ResultObjects ClosestPairVD::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Segment2D_ptr cpair;
    
    if (!SO.empty()) {
      if ((cpair = closest_pair_vd(extract_points(SO, GO), w, animation_depth, reference_depth)) != 0)
        result.push(cpair);
    }
      
    return result;
}
  
ResultObjects AllNearestNeighborsVD::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Segment2D_ptr) all_nn;
    Segment2D_ptr s;
    
    if (!SO.empty()) {
      all_nn = all_nearest_neighbors_vd(extract_points(SO, GO), w, animation_depth, reference_depth);
      forall(s, all_nn) {
        result.push(s);
      }
    }
      
    return result;
}

Algorithm* closest_pair_bf_link() {
    return new ClosestPairBF();
}

Algorithm* all_nn_bf_link() {
    return new AllNearestNeighborsBF();
}

Algorithm* closest_pair_dc_link() {
    return new ClosestPairDC();
}

Algorithm* all_nn_dc_link() {
    return new AllNearestNeighborsDC();
}

Algorithm* closest_pair_vd_link() {
    return new ClosestPairVD();
}

Algorithm* all_nn_vd_link() {
    return new AllNearestNeighborsVD();
}


