/*
    Copyright (C) 1992-1994
                    Laerte F. Morgado

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

//
//
//   Pslg.h - Definicao das classes Pslg e GraphicPslg
//            Essas classes implementam uma  classe externa
//            que permite a manipulacao de PSLGS: Planar
//            Straight-Line Graphs.


#ifndef _PslgDEFINED
#define _PslgDEFINED


#include "global.h"
#include "tools.h"
#include "utilities.h"
#include "World.h"
#include "Obj.h"
#include "ExternalObj_id.h" 
#include "half_edge.h"
#include "Segment2D.h"
#include "basic.h"
#include "List.h"

#ifndef _address_list
#define _address_list
declare(list,address)
#endif ~_address_list     
  
#ifndef _Face_ptr_list
#define _Face_ptr_list
declare(list,Face_ptr)
#endif ~_Face_ptr_list

  
// forward declaration
class GraphicPslg;
  
class Pslg:public Obj, public GWB
{
  public:
    Pslg(char *s = "PSLG", ObjType t = Generic_t, bool v = false):Obj(s,t,v) {;}
    Pslg( Pslg& s):Obj(s.what_is(), s.stype, s.volati)
       {s.replicate2(*this);}
    ~Pslg();
    
    GraphicObj* new_GraphicObj();

    Obj* reproduct()  {
	Pslg* pslg;
	pslg = new Pslg;
	this->replicate2(*pslg);
	return pslg;
//	return new Pslg(*this);
  }

    ObjType type()  { return Pslg_t;}

    void translate(double to_x, double to_y);
    void print()  { cout << " This Object is a Pslg" << endl;}
    void unfill_handles(World* w, bool show_handle = true) ;
    void draw_handles(World* w,GC gc,bool highlight = false,bool show_handle = true) ;
    void draw(World *w,GC gc) ;
    bool select_test(double x, double y) ;
    bool select_test(double x1, double y1, double x2, double y2) ;

    double distance_to_point(double x, double y) ;


    /* Euler operators */

    Face_ptr geo_mef(Id solid,Id face,Id v1,Id v2,Id new_face);
    void geo_smev( Id sno, Id f1, Id v1, Id v4, float x, float y, float z = 0 );
    void kill_rays(Vertex_ptr v,list(address)& t,Face_ptr& face);
    
    /* Load & save ... */
    void save(FILE* file);
    void load(FILE* file);

    /* Replication */
    void  replicate(Pslg& pslg);
    void  replicate2(Pslg& pslg);
};


/*
 *
 *
 *  GraphicPslg.h
 *
 *
 */


class GraphicPslg: public GraphicObj
{
  public:
    GraphicPslg():GraphicObj(new Pslg())
      {;}
    
    GraphicPslg(Pslg *s):GraphicObj(s)
      {;}
    
    ~GraphicPslg() {}
    
};

typedef Pslg* Pslg_ptr;
typedef GraphicPslg* GraphicPslg_ptr;

#endif ~_PslgDEFINED;








