/*
    Copyright (C) 1992-1994
                    Laerte F. Morgado

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 *   Pslg_tools.h:  Contem a declaracao de ferramentas que trabalham
 *                  com a estrutura Pslg. Em particular, com a estrutura
 *                  de dados "half-edge" de suporte.
 *
 *  Laerte F. Morgado    01/03/92        arquivo criado.
 *
 *
 */


#ifndef _Pslg_toolsDEFINED
#define _Pslg_toolsDEFINED

#include <xview/base.h>
#include "half_edge.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "Pslg_geotools.h"
#include "basic.h"
#include "List.h"
#include <iostream.h>
#include "Pslg.h"


#define X_VERTEX_GRAVITY _HX
#define Y_VERTEX_GRAVITY _HY
#define VERTEX_GRAVITY (sqrt(X_VERTEX_GRAVITY*Y_VERTEX_GRAVITY))

enum Pslg_inter_type { NO_CR, ONE_IN_CR, ONE_VERTEX_CR,MANY_CR }; 

class Pslg_tools_inter
{
  public:
    Vertex*     vertex;
    HalfEdge*   he;
    Point2D p;
    Pslg_inter_type status; 
};

typedef Pslg_tools_inter * Pslg_tools_inter_ptr;


declare (list,Pslg_tools_inter_ptr);

extern int vertex_gravity(Solid* solid,double& x,double& y,Vertex*& vertex,double X_gravity = X_VERTEX_GRAVITY, double Y_gravity = Y_VERTEX_GRAVITY);

long intersect(Solid* solid, double x1,double y1,double x2,double y2,InterType mode1=BOTH_CLOSED,InterType mode2=FIRST_OPEN);

extern void find_face(Face*& face, Vertex* vertex, double x, double y);

extern void Pslg_intersect(Pslg_tools_inter& it, HalfEdge& pslg_edge, Segment2D& s);

extern void kill_Pslg_tools_list(list(Pslg_tools_inter_ptr)& l);

extern void show_Pslg_tools_list(list(Pslg_tools_inter_ptr)& l);     

extern void copy_Pslg_tools_list(list(Pslg_tools_inter_ptr)& old_list,list(Pslg_tools_inter_ptr)& new_list);

extern bool All_Pslg_intersect(list(Pslg_tools_inter_ptr)& l, Solid*& solid,Segment2D& s);
     
extern bool Pslg_apply_Euler(Pslg& pslg,Id solid_id,list(Pslg_tools_inter_ptr)& l);

extern void Pslg_apply_graphical_Euler(list(Pslg_tools_inter_ptr)& l,World* world,DrawMode dm);     

extern void get_face_polygon(Vertex* vi,Vertex* vf,HalfEdge*& he1,HalfEdge*& he2,Face* face);

extern double Angle(HalfEdge_ptr edge,Vertex_ptr vet);

extern HalfEdge_ptr FirstInComing(Vertex_ptr vet);

extern HalfEdge_ptr InComing(Vertex_ptr vet,HalfEdge_ptr edge);

extern HalfEdge_ptr FirstOutComing(Vertex_ptr vet);

extern HalfEdge_ptr OutComing(Vertex_ptr vet,HalfEdge_ptr edge);

extern void find_horiz(Vertex_ptr v,HalfEdge_ptr& left,HalfEdge_ptr& right);

extern void find_min_max_x_y( Pslg& pslg,Vertex_ptr& minx,Vertex_ptr& miny,Vertex_ptr& maxx,Vertex_ptr& maxy);

extern long vertex_degree(Vertex_ptr vertex);

extern Face_ptr find_external_face( Pslg& pslg);

#endif ~_Pslg_toolsDEFINED;






