/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Rectangle.C - Rectangle & GraphicRectangle methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Obj.h"
#include "Rectangle.h"
#include <assert.h>

GraphicObj* Rectangle::new_GraphicObj() {
    return new GraphicRectangle(this);
}

bool GraphicRectangle::where_draw_string (double &x, double &y) {
    Point2D p1 (((Rectangle *)obj)->P1());
    Point2D p2 (((Rectangle *)obj)->P2());
    x = (p1.X()+p2.X())/2;
    y = (p1.Y()+p2.Y())/2;
    return true;
}

// Metodos necessarios para "Input Tools"
    
bool Rectangle::point_inside(double x, double y)  {
    double x1, y1, x2, y2;
    x1 = p1.X();
    y1 = p1.Y();
    x2 = p2.X();
    y2 = p2.Y();
    regularize_coords (x1, y1, x2, y2);
    return x1<x && x<x2 && y1<y && y<y2;
}

void Rectangle::bounding_box(double& x1, double& y1, double& x2, double& y2)  {
    x1 = p1.X();
    y1 = p1.Y();
    x2 = p2.X();
    y2 = p2.Y();
}
    
     
list(Obj_ptr) Rectangle::get_segments()  {
    list(Obj_ptr) S;
    Segment2D_ptr s_ptr;
    
    set_handles();
    s_ptr = new Segment2D(p1,p3);S.push(s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(p1,p4);S.push(s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(p2,p3);S.push(s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(p2,p4);S.push(s_ptr);assert(s_ptr);
    return S;
}

void Rectangle::unfill_handles(World *w, bool show_handle)  {
    if (show_handle) {
      set_handles();
      p1.unfill_handles(w);
      p2.unfill_handles(w);
      p3.unfill_handles(w);
      p4.unfill_handles(w);
    }
}

void Rectangle::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (show_handle) {
      set_handles();
      p1.draw_handles(w, gc, highlight);
      p2.draw_handles(w, gc, highlight);
      p3.draw_handles(w, gc, highlight);
      p4.draw_handles(w, gc, highlight);
    }
}

void Rectangle::draw(World *w, GC gc)  {
    w->DrawRectangle(gc, p1.X(), p1.Y(), p2.X(), p2.Y());
}

list(Obj_ptr) Rectangle::get_points()  {
    list(Obj_ptr) Objp;
    Point2D_ptr p1,p2,p3,p4;
    
    set_handles();
    p1 = new Point2D(P1()); assert(p1);
    p2 = new Point2D(P2()); assert(p2);
    p3 = new Point2D(P3()); assert(p3);
    p4 = new Point2D(P4()); assert(p4);

    Objp.append(p1);
    Objp.append(p2);
    Objp.append(p3);
    Objp.append(p4);

    return Objp;
}

/*
 *
 *
 */

// cast macro

#define rectangle ((Rectangle *)obj)

// para desenho de um retangulo centrado em um ponto (igual ao usado em
// Ellipse.C - talvez alguem bondoso no futuro coloque estas funcoes
// iguais em um modulo externo e documentar)

static double init_X, init_Y;
void set_coord_rect(double current_X, double current_Y,
		    double &x1, double &y1, double &x2, double &y2) {
    x1 = (init_X <= current_X) ? init_X : current_X;
    y1 = (init_Y <= current_Y) ? init_Y : current_Y;
    x2 = (init_X > current_X) ? init_X : current_X;
    y2 = (init_Y > current_Y) ? init_Y : current_Y;
    if (current_X > init_X)
      x1 -= fabs(x2 - x1);
    else
      x2 += fabs(x2 - x1);
    if (current_Y > init_Y)
      y1 -= fabs(y2 - y1);
    else
      y2 += fabs(y2 - y1);
}
						
										
// inherited methods redefinition

void GraphicRectangle::int_start(World *, double initial_X, double initial_Y, Int_Mode mode) {
    if (mode == MiddleButton) {
      init_X = initial_X;
      init_Y = initial_Y;
    }
    rectangle->set(initial_X, initial_Y, initial_X, initial_Y);
}

void GraphicRectangle::int_change(World *, double new_X, double new_Y, Int_Mode mode) {
    if (mode == LeftButton)
      rectangle->set(rectangle->P1().X(), rectangle->P1().Y(), new_X, new_Y);
    else {
      double x1, y1, x2, y2;
      set_coord_rect(new_X, new_Y, x1, y1, x2, y2);
      rectangle->set(x1, y1, x2, y2);
    }
}

void GraphicRectangle::int_end(World *w, double final_X, double final_Y, Int_Mode mode) {
    int_change(w, final_X, final_Y, mode);
}

void GraphicRectangle::resize_start(World *, double X, double Y, Int_Mode mode) {
    if (mode == MiddleButton) {
      init_X = (rectangle->P1().X() + rectangle->P2().X()) / 2;
      init_Y = (rectangle->P1().Y() + rectangle->P2().Y()) / 2;
    }
    if (rectangle->P1().select_test(X, Y))
      rectangle->set(rectangle->P2(), rectangle->P1());
    else
      if (rectangle->P3().select_test(X, Y))
        rectangle->set(rectangle->P4(), rectangle->P3());
      else
        if (rectangle->P4().select_test(X, Y))
          rectangle->set(rectangle->P3(), rectangle->P4());
}

void GraphicRectangle::resize_change(World *w, double X, double Y, Int_Mode mode) {
    int_change(w, X, Y, mode);
}

void GraphicRectangle::resize_end(World *w, double X, double Y, Int_Mode mode) {
    resize_change(w, X, Y, mode);
}


