/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Segment2D.C - Segment2D & GraphicSegment2D methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Obj.h"
#include "Segment2D.h"
#include "Line2D.h"
#include "Ray2D.h"

GraphicObj* Segment2D::new_GraphicObj() {
    return new GraphicSegment2D(this);
}

bool into ( Point2D& p1, Point2D& p2, Point2D& p3) {
    if (p1==p3) return true;
    return  (lexico_Xcomp (p1,p3)!=lexico_Xcomp (p2,p3));
}

bool Segment2D::point_in ( Point2D& p) {
    Point2D p1=P1(), p2=P2();
    return (p.colinear(p1,p2) && into (p1,p2,p));
}

bool Segment2D::intersect( Segment2D s) {

    if (is_line()) {
      Line2D l(p1, p2);
      return l.intersect(s.p1, s.p2);
    }

    if (is_ray()) {
      Ray2D r;
      if (p1.at_infinity())
        r.set(p2, p1);
      else
        r.set(p1, p2);
      return r.intersect(s.p1, s.p2);
    }  

    /****** ALTERNATIVA *****      
    Point2D p = intersection(s);
    if (p.at_infinity()) {
	Point2D p1=P1(),p2=P2(),sp1=s.P1(),sp2=s.P2();
	if (p1.colinear(p2,sp1) && p1.colinear(p2,sp2))
	  return (into(p1,p2,sp1) || into(p1,p2,sp2) || into(sp1,sp2,p1));
	else
	  return false;
    }
    else
      return true;      
    *************************/
    
    double a1, a2, b1, b2;
    CompType comp1, comp2;
    Point2D p1=P1(),p2=P2(),sp1=s.P1(),sp2=s.P2();

    if (p1.colinear(p2,sp1) && p1.colinear(p2,sp2))
      return (into(p1, p2, sp1) || into (p1, p2, sp2) || into (sp1, sp2, p1));

    a1 = (p2.X() - p1.X()) * (sp1.Y() - p1.Y()) - (sp1.X() - p1.X()) * (p2.Y() - p1.Y());
    a2 = (p2.X() - p1.X()) * (sp2.Y() - p1.Y()) - (sp2.X() - p1.X()) * (p2.Y() - p1.Y());
  
    b1 = (sp2.X() - sp1.X()) * (p1.Y() - sp1.Y()) - (p1.X() - sp1.X()) * (sp2.Y() - sp1.Y());
    b2 = (sp2.X() - sp1.X()) * (p2.Y() - sp1.Y()) - (p2.X() - sp1.X()) * (sp2.Y() - sp1.Y());
	  
    comp1 = comp(a1 * a2, 0, EPSLON);
    comp2 = comp(b1 * b2, 0, EPSLON);
	  
    if ((comp1 == Less || comp1 == Equal) && (comp2 == Less || comp2 == Equal))
      return true;

    return false;
}

bool Segment2D::intersect( Point2D a,  Point2D b) {
    return intersect(Segment2D(a, b));
}



/*
 * Metodo alterado p/ detectar a interse,c~ao entre os segmentos
 * quando estes s~ao paralelos, mas possuem uma 'unica intersec,c~ao.
 * Gon, 03/02/93.
 */

/* Alguns testes realizados mostram que, devido aos c`alculos de precis~ao
 * finita realizados, algumas vezes (raras vezes) a interse,c~ao
 * 'e calculada com um erro maior que EPSON, da ordem de 10e-2.
 * Gon, 03/02/93.
 */

Point2D Segment2D::intersection( Segment2D s) {
    if (is_line()) {
	Line2D l(p1, p2);
	return l.intersection(s.p1, s.p2);
    }
    
    if (is_ray()) {
	Ray2D r;
	if (p1.at_infinity())
	  r.set(p2, p1);
	else
	  r.set(p1, p2);
	return r.intersection(s.p1, s.p2);
    }  

    Point2D res;
    double v1, v2, det, detinv;
    double xlk, ylk, xnm, ynm, xmk, ymk; 
    CompType cv10, cv11, cv20, cv21;
    
    xlk = P2().X() - P1().X();
    ylk = P2().Y() - P1().Y();
    xnm = s.P2().X() - s.P1().X();
    ynm = s.P2().Y() - s.P1().Y();
    xmk = s.P1().X() - P1().X();
    ymk = s.P1().Y() - P1().Y();
    
    det = (xnm * ylk) - (ynm * xlk);
    if (zero(det)) {
	// parallel line segments
	if (p1 == s.p1 || p1 == s.p2)
	  res = p1;
	else if (p2 == s.p1 || p2 == s.p2)
	  res = p2;
	else	  
	  res.set_at_infinity(0, 0);	
    }
    else {
	detinv = 1/det;
	v1 = ((xnm * ymk) - (ynm * xmk)) * detinv;
	v2 = ((xlk * ymk) - (ylk * xmk)) * detinv;
    
	cv10 = comp(v1, 0); cv11 = comp(v1, 1);
	cv20 = comp(v2, 0); cv21 = comp(v2, 1);
    
	if (((cv10 == Equal || cv10 == Greater) && (cv11 == Equal || cv11 == Less)) &&
          ((cv20 == Equal || cv20 == Greater) && (cv21 == Equal || cv21 == Less))) {
	    res.setXY(P1().X() + (xlk * v1), P1().Y() + (ylk * v1));
	}      
	else
	  res.set_at_infinity(0, 0);	// no bounded intersection
    }

    return res;
}

Point2D Segment2D::intersection( Point2D a,  Point2D b) {
    return intersection(Segment2D(a, b));
}

void Segment2D::unfill_handles(World *w, bool show_handle)  {
    if (show_handle) {
      p1.unfill_handles(w);
      p2.unfill_handles(w);
    }
}

void Segment2D::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (show_handle) {
      p1.draw_handles(w, gc, highlight);
      p2.draw_handles(w, gc, highlight);
    }
}

void Segment2D::draw(World *w, GC gc)  {
    if (is_line()) {
      Line2D l(p1, p2);
      l.draw(w, gc);
      return;
    }

    if (is_ray()) {
      Ray2D r;
      if (p1.at_infinity())
        r.set(p2, p1);
      else
        r.set(p1, p2);
      r.draw(w, gc);
      return;
    }  

    w->DrawLine(gc, p1.X(), p1.Y(), p2.X(), p2.Y());
}

double Segment2D::distance_to_point(double x, double y)  {
    if (is_line()) {
      Line2D l(p1, p2);
      return l.distance_to_point(x, y);
    }

    if (is_ray()) {
      Ray2D r;
      if (p2.at_infinity())
        r.set(p1, p2);
      else
        r.set(p2, p1);
      return r.distance_to_point(x, y);
    }  

    // as extremidades do segmento sao k e l, o ponto dado e' j
    double xkj = p1.X() - x;
    double ykj = p1.Y() - y;
    double xlk = p2.X() - p1.X();
    double ylk = p2.Y() - p1.Y();
    double d = (xlk * xlk) + (ylk * ylk);
    
    if (d < EPSLON)
      return sqrt((xkj * xkj) + (ykj * ykj));	// segment ends coincide
    else {
      double t = -((xkj * xlk) + (ykj * ylk)) / d;
      t = min(max(t, 0), 1);
      double xfac = xkj + (t * xlk);
      double yfac = ykj + (t * ylk);
      return sqrt((xfac * xfac) + (yfac * yfac));
    }
}

void Segment2D::translate_handle_at(double x, double y, double to_x, double to_y) {
    if (p1.select_test(x, y))
      p1.translate(to_x, to_y);
    else
      if (p2.select_test(x, y))
        p2.translate(to_x, to_y);
  
    if (p1.equalX(p2))
      subtype(Vertical_t);
    else
      if (p1.equalY(p2))
        subtype(Horizontal_t);
      else
       subtype(Generic_t);
}

list(Obj_ptr) Segment2D::get_points()  {
    list(Obj_ptr) Objp;
    Point2D_ptr p1,p2;
    Point2D tmpP1 = P1(), tmpP2 = P2();
    p1 = new Point2D(tmpP1); assert(p1);
    p2 = new Point2D(tmpP2); assert(p2);
    Objp.append(p1);
    Objp.append(p2);
    return Objp;
}

/*
 *
 * GraphicSegment2D
 *
 *
 */

// cast macro

#define segment	((Segment2D *)obj)

static bool is_P1;

// inherited methods redefinition

void GraphicSegment2D::int_start(World *, double initial_X, double initial_Y, Int_Mode) {
    segment->set(initial_X, initial_Y, initial_X, initial_Y);
}

void GraphicSegment2D::int_change(World *, double new_X, double new_Y, Int_Mode) {
    segment->setP2(new_X, new_Y);
}

void GraphicSegment2D::int_end(World *, double final_X, double final_Y, Int_Mode) {
    segment->setP2(final_X, final_Y);
    if (segment->P1().equalX(segment->P2()))
      segment->subtype(Vertical_t);
    else
      if (segment->P1().equalY(segment->P2()))
        segment->subtype(Horizontal_t);
}

void GraphicSegment2D::resize_start(World *, double X, double Y, Int_Mode) {
    if (segment->P1().select_test(X, Y))
      is_P1 = true;
    else
      is_P1 = false;
}

void GraphicSegment2D::resize_change(World *, double X, double Y, Int_Mode) {
    if (is_P1)
      segment->set(X, Y, segment->P2().X(), segment->P2().Y());
    else
      segment->set(segment->P1().X(), segment->P1().Y(), X, Y);
}

void GraphicSegment2D::resize_end(World *w, double X, double Y, Int_Mode m) {
    resize_change(w, X, Y, m);
    if (segment->P1().equalX(segment->P2()))
      segment->subtype(Vertical_t);
    else
      if (segment->P1().equalY(segment->P2()))
        segment->subtype(Horizontal_t);
      else
        segment->subtype(Generic_t);
}

bool GraphicSegment2D::where_draw_string (double &x, double &y) {
    Point2D p1 (((Segment2D *)obj)->P1());
    Point2D p2 (((Segment2D *)obj)->P2());
    x = (p1.X()+p2.X())/2;
    y = (p1.Y()+p2.Y())/2;
    return true;
}
