/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * algorithms.h - header file de algorithms.C
 *
 *		  module to link with: /usr/lib/debug/mallocmap.o
 *
 *		  link flag -ll 
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    29.01.92  21:25 	Welson R. Jacometti  	Arquivo criado
 *    11.01.93  12:00   C'esar N. Gon           M'odos Externos
 *
 */

/*
 * IMPORTANTE:
 *
 *    O disparo de algoritmos externos e' feito unica e exclusivamente em
 *  dois modulos: algorithms.C(.h) e instantiation.C(.h) 
 *    Nestes modulos, antes da referencia ao algoritmo externo, DEVE ser
 *  feita a mudanca da rotina de tratamento de sinais, para que o usuario
 *  seja corretamente informado sobre problemas decorrentes a) do ambiente
 *  ou b) de um determinado algoritmo externo.
 *
 */
 
#ifndef _ALGORITHMS_DEFINED
#define _ALGORITHMS_DEFINED

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <malloc.h>
#include <xview/xview.h>
#include <xview/openmenu.h>
#include "global.h"
#include "World.h"
#include "geolab_ui.h"
#include "Algorithm.h"

EXPORT bool DISCARD_INPUT;
EXPORT bool UNSELECT_INPUT;
EXPORT bool OUTPUT_SELECTED;
EXPORT bool DYNAMIC_DISCARD_OUTPUT;
EXPORT bool DUMP_HEAP;
// EXPORT Menu_item LAST_MENU_ITEM;
EXPORT Algorithm* LAST_ALGORITHM;

typedef Algorithm* Algorithm_ptr;

typedef Algorithm_ptr (*link_function)();

/*
 * Nota: 
 *
 *	Na sua versao atual (release 2.0) o compilador de C++ da Sun 
 *      Microsystems gera, para o tipo de funcao abaixo
 *      
 *      Algorithm_ptr generic_link();
 *
 *      a seguinte traducao (que deve ser composta no dispatcher):
 *
 *      _generic_link__Fv
 *
 *      Se esta traducao nao for estavel, torna-se um problema neste projeto
 *      manter a funcao de traducao (algorithms.C) atualizada.
 *
 */
 
extern Attr_attribute INSTANCE;		// from the interface

String compose_external_algorithm_name(String function_name);
Menu menu_algorithms(caddr_t *, Xv_opaque);
void* open_dynamic_module(char *mname);
void close_dynamic_modules();
void menu_algorithms_dispatcher(Menu, Menu_item);
void dispatcher(World *, Algorithm *);
void exec_binder(World *, Algorithm *);

// suporte para composicao de algoritmos
bool compose_exec_start(World *w, Algorithm *alg);
void compose_exec(World *w, Algorithm *alg, bool di, bool ui, bool os, char *fn, char *mn);
void compose_exec_finish(World *w);

// suporte para dynamic move
bool last_algorithm_is_able_to_handle_current_objects();
char* last_algorithm_called_name();
char* last_algorithm_called_module_name();
ResultObjects last_algorithm_called(World *);
Algorithm_ptr last_algorithm();

// suporte para insercao de objetos criados externamente e interativamente
void install_externally_created_object(GraphicObj_ptr, Algorithm_ptr);

// suporte para o disable de opcoes no menu de algoritmos em funcao dos
// objetos selecionados

void enable_all_algorithms();
void disable_all_algorithms();
void disable_algorithms_by_animation();
void select_applyable_algorithms();

#endif ~_ALGORITHMS_DEFINED;





