/*
 * This code has been modified for compatibility with other
 * source codes of GeoLab.
 */

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+                                                                            +
+  LEDA  2.0.1                                                               +
+                                                                            +
+                                                                            +
+  Copyright (c) 1991  by  Max-Planck-Institut fuer Informatik               +
+  Im Stadtwald, 6600 Saarbruecken, FRG                                      +
+  All rights reserved.                                                      +
+                                                                            +
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#include "basic.h"

#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <sys/types.h>

#ifndef __TURBOC__
#include <sys/times.h>
#endif

LEDA_OP LEDA;

char *LEDA_OP::op_list = 0;

LEDA_OP::LEDA_OP()
{ op_list = getenv("LEDA_OP");
  if (op_list)
  { fprintf(stderr,
    "LEDA 2.0  (1990) (c) FB 14, Universitaet des Saarlandes, D-6600 Saarbruecken\n\n");
    fprintf(stderr,"OPTIONS = %s\n\n",op_list);
   }

  init_random();

 }


LEDA_OP::~LEDA_OP() 
{ if (op_list)
  { if (strcmp(op_list,"statistics")==0) print_statistics(); 
    printf("LEDA 2.0 - The End. \n"); 
    op_list = 0;
   }
 } 

//------------------------------------------------------------------------------
// Bool
//------------------------------------------------------------------------------

/*
const int false = 0;	// already defined in global.h & global.C
const int true  = 1;
*/

//------------------------------------------------------------------------------
// String
//------------------------------------------------------------------------------


char* str_dup(const char* p)
{ if (p==nil) error_handler(1,"str_dup: nil argument");
  char* q = new char[strlen(p)+1];
  strcpy(q,p);
  return q;
}

char* str_cat(const char* p1, const char* p2)
{ char* q = new char[strlen(p1)+strlen(p2)+1];
  strcpy(q,p1);
  strcat(q,p2);
  return q;
 }


/*
string::string(char* format, ...)
{
  va_list arg_list;
  Va_Start(arg_list,format);

  int ap[16];

  for(int i=0;i<16;i++) ap[i] = va_arg(arg_list,int);

  va_end(arg_list);

  char buf[1024];

  sprintf(buf,format,ap[0],ap[1],ap[2],ap[3],ap[4],ap[5],ap[6],ap[7],ap[8],ap[9],ap[10],ap[11],ap[12],ap[13],ap[14],ap[15]);

  p = new srep(buf);
}
*/

int  compare(string& x, string& y) 
{ return strcmp(x.p->s,y.p->s); 
 }

int   string::length()     const   
{ return strlen(p->s); 
 }

char& string::operator[](int i)
{ if (i<0 || strlen(p->s)<=i) error_handler(1,"string[]: index out of range");
  return p->s[i];
}


string& string::operator=(const char* s)
{ if (p->n > 1) { p->n--; p = new srep(str_dup("")); }
  delete p->s;
  p->s = str_dup(s);
  return *this;
}

string& string::operator=(const string& x)
{ x.p->n++;
  if (--p->n == 0) { delete p->s; delete p; }
  p = x.p;
  return *this;
}

string  string::operator+(const string& x)  const 
{ string res; 
  res.p->s = str_cat(p->s,x.p->s); 
  return res;
 }

string  string::operator+(const char* s1)   const 
{ string res;
  res.p->s = str_cat(p->s,s1); 
  return res;
 }

string& string::operator+=(const char* s1)  
{ *this = *this + s1; 
  return *this; 
 }

string& string::operator+=(const string& x) 
{ *this = *this + x; 
  return *this; 
 }

int operator==(const string& x, const char* s)   
{ return strcmp(x.p->s,s)==0; }

int operator==(const string& x, const string& y) 
{ return strcmp(x.p->s,y.p->s)==0; }

int operator!=(const string& x, const char* s)   
{ return strcmp(x.p->s,s)!=0; }

int operator!=(const string& x, const string& y) 
{ return strcmp(x.p->s,y.p->s)!=0; }

int operator<(const string& x, const string& y)  
{ return strcmp(x.p->s,y.p->s)<0; }

int operator>(const string& x, const string& y)  
{ return strcmp(x.p->s,y.p->s)>0; }

int operator<=(const string& x, const string& y) 
{ return strcmp(x.p->s,y.p->s)<=0; }

int operator>=(const string& x, const string& y) 
{ return strcmp(x.p->s,y.p->s)>=0; }


istream& operator>>(istream& in, string& x)
{ char buf[256];
  if (in >> buf) x = buf; 
  return in;
}

ostream& operator<<(ostream& out, const string& x) 
{ return out << x.p->s; }


string string::sub(int i, int j) const
{
  if (j>= strlen(p->s)) j = strlen(p->s) - 1;

  if (i < 0) i = 0;

  int l = j-i+1;

  if (l <= 0)  return string("");

  char* q = new char[l+1];
  strncpy(q,p->s+i,l);
  q[l] = '\0';
  return string(q);
}


int string::pos(string s1) const
{
  int l1 = s1.length();
  if (l1==0) return -1;

  char c = s1[0];

  char* q = p->s;
  while (*q)
    { while ( (*q) && (*q != c) ) q++;
      if ( strncmp(q,s1.p->s,l1)==0 ) break ;
      if (*q) q++;
     }
  return (*q) ? (q - p->s)/ sizeof(char) :  -1;
}

string string::insert(string s, int i) const
{ return head(i) + s + tail(i+1); }

string string::replace(string s1, string s2)const 
{ int p;
  if ( (p=pos(s1)) < 0 ) return *this;
  return head(p-1) + s2 + tail(p+s1.length());
 }


//------------------------------------------------------------------------------
// Real
//------------------------------------------------------------------------------

int compare(double& r1, double& r2) 
{ if (r1 < r2)  return -1;  
  if (r1 > r2)  return  1;  
  return 0; 
}

int compare(real& r1, real& r2) 
{ if (r1 < r2)  return -1;  
  if (r1 > r2)  return  1;  
  return 0; 
}

real::rrep::rrep(double x) 
{ 
  d = x; 
  n = 1;
}


real& real::operator=(const double x)
{ if (p->n > 1) { p->n--; p = new rrep(0); }
  p->d = x;
  return *this;
}

real& real::operator=(const real& x)
{ x.p->n++;
  if (--p->n == 0)  delete p;
  p = x.p;
  return *this;
}

istream& operator>>(istream& s, real& r) 
{ double d; s >> d; r = d; return s;}

ostream& operator<<(ostream& s, const real& r) 
{ return s << r.p->d; }

//------------------------------------------------------------------------------
// Memory Management
//------------------------------------------------------------------------------

const memory_word_size = 4;     // size of word  (bytes)
const memory_block_bytes = 4092; // size of block (bytes) 2^k - 4
const memory_block_words = 1023; // size of block (in words)
const memory_max_size = 64;     // maximal size of structures (words)

memory_elem_ptr memory_free_list[65];     //memory_max_size + 1
int             memory_total_count[65];   //memory_max_size + 1


memory_elem_ptr memory_allocate_block(int size)
{ // fills memory_free_list[size] with l new elements of size "size" 

  register memory_elem_ptr q;
  register memory_elem_ptr p;

  //if ((q=memory_elem_ptr(new char[memory_block_bytes]))==0)

  if ( (q=memory_elem_ptr(malloc(memory_block_bytes))) == 0 )
   { cout << form("memory_alloc: out of memory\n");
     print_statistics();
     abort();
    }

  memory_free_list[size] = q;

  int l = memory_block_words/size;
  memory_total_count[size] += l;

  p = q + (l-1)*size;
  p->next = 0;

  while (q < p) q = (q->next = q+size);


/*
  // rest
  int r;
  p = p+size;
  r = memory_block_bytes%(size * memory_word_size);
  if (r = r/memory_word_size)
  { p->next = memory_free_list[r];
    memory_free_list[r] = p;
    memory_total_count[r]++;
   }
*/

 return memory_free_list[size];
}

memory_elem_ptr allocate_words(int size)
{ memory_elem_ptr p = memory_free_list[size];
  if (p==0) p = memory_allocate_block(size);
  memory_free_list[size] = p->next;
  return p;
}

void deallocate_words(void* p, int size)
{ memory_elem_ptr(p)->next = memory_free_list[size];
  memory_free_list[size] = memory_elem_ptr(p);
 }

void deallocate_list(void* head,void* tail, int size)
{ memory_elem_ptr(tail)->next = memory_free_list[size];
  memory_free_list[size] = memory_elem_ptr(head);
 }

memory_elem_ptr Allocate_words(int size)
{ memory_elem_ptr p = memory_free_list[size];
  if (p==0) p = memory_allocate_block(size);
  memory_free_list[size] = p->next;
  cout << "Allocate   "<< int(p) << "(" << size << ")\n";
  return p;
}

void Deallocate_words(void* p, int size)
{ cout << "Deallocate  " << int(p) << "(" << size << ")\n";

  if (int(p)<=0 ) 
    error_handler(1,"deallocate: illegal pointer");  

  memory_elem_ptr(p)->next = memory_free_list[size];
  memory_free_list[size] = memory_elem_ptr(p);
}

void print_statistics()
{
  cout.flush();

  int total,free,used,i,size;
  int total_bytes=0, free_bytes=0, used_bytes=0, b;
  char* format="     | %3d    %5d   %5d   %5d   %8d bytes |\n";     

  printf("\n");
  printf("     +-----------------------------------------------+\n");
  printf("     | size    free    used   total      space       |\n"); 
  printf("     +-----------------------------------------------+\n");

  for (i=1;i<=memory_max_size;i++)
    if ((total = memory_total_count[i]) || memory_free_list[i])
    { memory_elem_ptr p = memory_free_list[i];
      size = memory_word_size*i;
      free = 0;
      while (p) { free++; p = p->next; }
      b = total*size; 
      used  = total - free;
      free_bytes  = free_bytes  + free*size;
      used_bytes  = used_bytes  + used*size;
      total_bytes = total_bytes + b;
      printf(format,i,free,used,total,b);
     }

 printf("     +-----------------------------------------------+\n");
 printf("     |                   total space: %8d kb    |\n",total_bytes/1024);
 printf("     +-----------------------------------------------+\n");
 printf("\n");
 fflush(stdout);

}



//------------------------------------------------------------------------------
// Error Handling
//------------------------------------------------------------------------------


PEH error_handler = default_error_handler;

void default_error_handler(int i, char* s)
{ if (i==0) 
   cout << "(warning) " << s << "\n";
  else 
   { cout << "ERROR "<< s << "\n";
     cout.flush();
     abort();
    }
}

PEH set_error_handler(PEH handler)
{ PEH old = error_handler;
  error_handler = handler;
  return old;
}

//------------------------------------------------------------------------------
// usefull functions
//------------------------------------------------------------------------------

void init_random(int seed)
{ long l = long(seed);
  if (l==0) time(&l);
  srand(int(l));
}

real rrandom()                
{ return real(rand())/Infinity; }

unsigned   random(unsigned a, unsigned b)   
{ return  unsigned(a+(b-a+1)*(double(rand())/Infinity)); }

int   random(int a, int b)   
{ return  int(a+(b-a+1)*(double(rand())/Infinity)); }



#ifndef __TURBOC__

double used_time()
{ tms x;
  times(&x);
  return  double(x.tms_utime)/60;
}

double used_time(real& T)
{ double t = T;
  tms x;
  times(&x);
  T =  double(x.tms_utime)/60;
  return  double(T)-t;
}

double used_time(double& T)
{ double t = T;
  tms x;
  times(&x);
  T =  double(x.tms_utime)/60;
  return  double(T)-t;
}

float used_time(float& T)
{ float t = T;
  tms x;
  times(&x);
  T =  float(x.tms_utime)/60;
  return  float(T)-t;
}

void print_time(string s)
{
  tms x;
  times(&x);
  float ut = float(x.tms_utime)/60;
  float st = float(x.tms_stime)/60;
  cerr << s;
  cerr << form("user time: %.2f sec   system time: %.2f sec \n",ut,st);
}

#endif

//------------------------------------------------------------------------------
// Input/Ouput
//------------------------------------------------------------------------------


file_ostream::file_ostream(string fname) : ostream(&fb)
{ if (fb.open(~fname,output)==0)
  error_handler(1,"cannot open file ");
 }

void file_ostream::close() { fb.close(); clear(); }

bool file_ostream::open(string fname)
{ close();
  return (fb.open(~fname,output) !=0);
 }



file_istream::file_istream(string fname) : istream(&fb)
{ if (fb.open(~fname,input)==0)
  error_handler(1,"cannot open file");
 }

void file_istream::close() { fb.close(); clear(); }

bool file_istream::open(string fname)
{ close();
  return (fb.open(~fname,input) !=0);
 }

string_ostream::string_ostream() : ostream(2048,buf) {}

string string_ostream::str() 
{ return buf; }


string_istream::string_istream(string s) : istream(2048,buf) 
{ for(int i=0;i<2048;i++) buf[i] = 0; 
  strncpy(buf,~s,2048); 
 }

string_istream::string_istream(char* s) : istream(2048,buf) 
{ for(int i=0;i<2048;i++) buf[i] = ' '; 
  strncpy(buf,s,2048); 
 }

string_istream::string_istream(int argc, char** argv) : istream(2048,buf) 
{ for(int i=0;i<2048;i++) buf[i] = 0; 
  char* b = buf;
  int rest = 2048;
  for(i=0;i<argc;i++) 
  { int l = strlen(argv[i]) + 1;
    strncpy(b,argv[i],rest);
    rest -= l;
    b += l;
    b[-1] = ' ';
   }
 }


#ifndef __TURBOC__

#ifdef __GNUG__

cmd_ostream::cmd_ostream(string cmd) : ostream(popen(~cmd,"w")) {}
cmd_istream::cmd_istream(string cmd) : istream(popen(~cmd,"r")) {}

#else

cmd_ostream::cmd_ostream(string cmd) : b(popen(~cmd,"w")), ostream(&b) {}
cmd_istream::cmd_istream(string cmd) : b(popen(~cmd,"r")), istream(&b) {}

#endif

#endif

  
int Yes(string s)
{ char answer = read_char(s);
  return ((answer == 'y') || (answer == 'Y'));
}

int read_int(string s)
{ int answer;
  char c;
  int success = 0;
  while (!success)
  { cout << s;
    cin >> answer;
    if (!cin) { cin.clear();
                cin.get(c);
                cout << form("read_int: illegal input \"%c\"\n",c);
                while (c!='\n') cin.get(c); //skip rest of line
               }
    else  success=1;
   }
  cin.get(c);   // eat '\n'

  return answer;
}

char read_char(string s)
{ char c;
  cout << s;
  cin.get(c);
  if (c!='\n') read_line(cin);
  return c;
}

real read_real(string s)
{ double answer;
  cout << s;
  cin >> answer;
  read_line(cin);
  return answer;
}

string  read_line(istream& s)
{ char buf[256];
  char* p = buf;

  while (s.get(*p))
  { if (*p == '\n') break;
    p++;
   } 
  *p = '\0';
  return string(buf);
}

string read_string(string s)
{ cout << s;
  return read_line(cin); 
 }

int     Yes()              { return Yes(""); }
int     read_int()         { return read_int(""); }
char    read_char()        { return read_char(""); }
real    read_real()        { return read_real(""); }
string  read_string()      { return read_string(""); } 
string  read_line()        { return read_line(cin); }





