/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * draw_arrow.C - implementacao das funcoes para desenho de Arrow2D.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.02.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "draw_arrow.h"

icon_bits draw_arrow_icon[] = {
#include "ICONS/draw_arrow.icon"
};

bool draw_arrow_events(World *world, Xv_Window, Event *event) {
    static GraphicArrow2D *current_arrow = 0;
    static Int_Mode mode = LeftButton;
    
    double X = world->virtual_xcoord(event_x(event));
    double Y = world->virtual_ycoord(event_y(event));
    
    switch (event_action(event)) {
      case ACTION_SELECT:
      case ACTION_ADJUST:
        if (event_is_down(event) && !current_arrow) {
          if (event_action(event) == ACTION_SELECT)
            mode = LeftButton;
          else
            mode = MiddleButton;
          current_arrow = new GraphicArrow2D(); 
          current_arrow->set_all(world, FOREGROUND, BACKGROUND, 0, 0, HandlesOn);
          current_arrow->int_start(world, X, Y, mode);
          current_arrow->draw(world, XorMode);
        }
        else
          if (current_arrow) {
            current_arrow->draw(world, XorMode);
            current_arrow->int_end(world, X, Y, mode);
            current_arrow->set_line_attr(world, LINE_WIDTH, LINE_STYLE);
            current_arrow->draw(world, CopyMode);
            EO_install_externally_created_object(current_arrow);
            current_arrow = 0;
          }
        break;
        
      case LOC_DRAG:
        if (current_arrow) {
          current_arrow->draw(world, XorMode);
          current_arrow->int_change(world, X, Y, mode);
          current_arrow->draw(world, XorMode);
        }
        break;
    }
    
    // retorna true APENAS se for necessario CONSUMIR o evento
    return false;
}


//
// ExternalObjectMode methods & functions...
//

icon_bits *draw_arrow::icon() {return draw_arrow_icon;}

void draw_arrow::activator(World *w) {
    w->add_event_proc(draw_arrow_events);
    w->add_event_proc(fixed_movement_events);
}

Algorithm* draw_arrow_link() {
    return new draw_arrow();
}






