/*
    Copyright (C) 1992-1994
		    Juliano L. de Oliveira

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

#ifndef	goodies_file_HEADER
#define	goodies_file_HEADER

//
// goodies_file.h 
//

// The file_window is an auxiliar window used for file selection, as it
// 	allows navigation on the directories of the file system.
//
// In order to use the file_window facility, you must:
//
// a. use the command "#include "/home/pos/juliano/PUB/goodies_file.h"" in
//	your application;
//	
// b. create a function ("file_function") that will be called when the desired
//	file is selected in the file_window. The parameters for the
//	file_function are described below;
//
// c. call the function "goodies_choose_file", with the parameters listed
//	below. This function will create the file_window and will manage the
//	user actions on this window. When the user selects a file in the
//	file window, the goodies_choose_file function automatically destroys
//	the file_window and calls your file_function.
//
// d. link with your application the files goodies_file.o and
//	goodies_file_ui.o;


#include <sys/param.h>
#include <xview/xview.h>

// The following instruction defines a pointer to a void function with two
//	parameters.
// This function will be executed when the file is selected in the file_window.
// The first parameter is the selected file name and the second is the
// 	selected file path. Both parameters are pointers to characters.

typedef void (*file_function) (char*, char*);

// The parameters for goodies_choose_file function are:
//
// 1. the handler of a base window that will be the owner of the file_window;
// 2. a flag indicating whether the creation of new files is allowed
//    (0=don't allow;1=allow);
// 3. the initial directory to be presented in the file_window;
// 4. the "magic-number" of the file. This is a string of four characters
//	that will be compared with the first four characters of the files
//	in the directory. Only the matching files and the subdirectories
//	will be presented in the file_window. An "*" can be used to indicate
//	that any character will be accepted as matching the magic-number.
//	Thus,"****" will cause all files in the directory to be presented;
// 5. the title to be presented in the file_window header;
// 6. the label of the text field (usually something like "Load File", or
//    "Save File as"));
// 7. the label of the "ok" button;
// 8. the label of the "cancel" button;
// 9. the pointer of the function to be executed when a file is selected.

extern void goodies_choose_file(Xv_opaque, int, char *, char[4], char *,
	char *, char *, char *, file_function);

//
// file_window class declaration.
//

class goodies_file_window_objects {

public:
	Xv_opaque			file_window;
	Xv_opaque			file_panel;
	Xv_opaque			file_textfield;
	Xv_opaque			file_list;
	Xv_opaque			file_ok;
	Xv_opaque			file_cancel;
	Xv_opaque			file_msg1;
	char 				file_dir_name[MAXPATHLEN];
	char				file_magic_number[4];
	char				file_fd_file_name[MAXPATHLEN];
	char				file_fd_path_name[MAXPATHLEN];
	int				file_fd_file_type;
	file_function			file_file_func;
	int				file_new_file;

	virtual Xv_opaque	file_window_create(Xv_opaque, char *);
	virtual Xv_opaque	file_panel_create();
	virtual Xv_opaque	file_textfield_create(char *);
	virtual Xv_opaque	file_list_create();
	virtual Xv_opaque	file_ok_create(char *);
	virtual Xv_opaque	file_cancel_create(char *);
	virtual Xv_opaque	file_msg1_create();
};

int goodies_file_window_on_screen();
void goodies_file_window_show_true();
void goodies_file_window_show_false();

#endif


