/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * load_save.C - modulo para armazenamento de objetos em memoria secundaria.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.09.91  18:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "load_save.h"
#include "String.h"
#include "Obj.h"
#include "basic.h"
#include "List.h"
#include "objects.h"
#include "instantiation.h"

#ifndef _GraphicObj_ptr_list
#define _GraphicObj_ptr_list
declare(list,GraphicObj_ptr)
#endif ~_GraphicObj_ptr_list;

// variaveis locais auxiliares 
char CURRENT_DIRECTORY_NAME_AUX[MAXPATHLEN];

// variaveis exportaveis
char *CURRENT_DIRECTORY_NAME = getwd(CURRENT_DIRECTORY_NAME_AUX);
char *CURRENT_FILE_NAME = '\0'; 

bool file_exists(char *directory, char *filename) {
    FILE *f;
    String fname;

    if (*directory == '\0')
      fname = String(filename);
    else
      fname = String(directory) + String("/") + String(filename);

    if (f = fopen(fname.value(), "r")) {
      fclose(f);
      return true;
    }
    else
      return false;
}

GraphicObj *load_object(FILE *f, World *w) {
    ObjType t;
    Obj_ptr obj;
    GraphicObj_ptr graphic_obj = 0;
    char function_name[MAXPATHLEN];
    char module_name[MAXPATHLEN];
    
    if (fscanf(f, "%d\n", &t) != -1) {
      if (geolab_knows_object(t))
      	obj = geolab_instantiation(t);
      else {
        if (fscanf(f, "%s %s\n", function_name, module_name) == -1)
          return 0;
        obj = external_instantiation(t, function_name, module_name);
      }
      
      graphic_obj = obj->new_GraphicObj();
      if (!geolab_knows_object(t))
        graphic_obj->set_external(function_name, module_name);
      graphic_obj->load(f, w);
    }  
    
    return graphic_obj;
}

bool load(World *w, char *directory, char *filename) {
    FILE *f;
    String fname;
    if (*directory == '\0')
      fname = String(filename);
    else
      fname = String(directory) + String("/") + String(filename);

    GraphicObj_ptr graphic_obj = 0;
    
    if (!filename || strlen(filename) == 0 || !(f = fopen(fname.value(), "r")))
      return false;

    while (graphic_obj = load_object(f, w)) {
      if (graphic_obj->selected())
        objects_insert_and_select(graphic_obj);
      else
        objects_insert(graphic_obj);
    }
    
    fclose(f);
    return true;
}

bool save(char *directory, char *filename) {
    FILE *f;
    String fname;
    if (*directory == '\0')
      fname = String(filename);
    else
      fname = String(directory) + String("/") + String(filename);

    list(GraphicObj_ptr) Objects;
    GraphicObj_ptr obj;

    if (!filename || strlen(filename) == 0 || !(f = fopen(fname.value(), "w")))
      return false;

    Objects = objects_get_graphic_objects();
    
    forall(obj, Objects) {
      int res;
      if (geolab_knows_object(obj->type()))
        res = fprintf(f, "%d\n", obj->type());
      else
        res = fprintf(f, "%d\n%s %s\n", obj->type(), obj->get_external_function(), obj->get_external_module());
      if (res == EOF) {
      	Objects.init_iterator();
      	return false;
      }
      obj->save(f);
    }
    
    fclose(f);
    return true;
}






