/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * objects.h - manutencao dos objetos geometricos manipulados pelo GeoLab.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    12.01.92  18:25 	Welson R. Jacometti  	Arquivo criado
 *
 */
 
#ifndef _OBJECTS_DEFINED
#define _OBJECTS_DEFINED

#include "global.h"
#include "basic.h"		// LEDA module
#include "List.h"		// LEDA module
#include "World.h"
#include "Obj.h"

// variaveis exportadas

EXPORT bool HandlesOn;

#ifndef _Obj_ptr_list
#define _Obj_ptr_list
declare(list,Obj_ptr)
#endif ~_Obj-ptr_list

#ifndef _GraphicObj_ptr_list
#define _GraphicObj_ptr_list
declare(list,GraphicObj_ptr)
#endif ~_GraphicObj_ptr_list;

#ifndef _list_item_list
#define _list_item_list
declare(list,list_item)
#endif ~_list_item_list

void objects_insert(GraphicObj *obj);
void objects_insert_and_select(GraphicObj *obj);
void objects_insert_external(GraphicObj *obj, char *function = 0, char *module = 0);
void objects_clear();
bool objects_empty();
bool objects_selected_empty();
void objects_repaint(World *w);
void objects_repaint_all_but(GraphicObj *not_to_repaint, World *w);
void objects_repaint_unselected(World *w);
void objects_refresh(World *w);
void objects_set_handle(World *w, bool all, bool show);

bool objects_select_one_left_button(World *w, double x, double y);
bool objects_select_one_middle_button(World *w, double x, double y);

void objects_select_all(World *w);
void objects_select(World *w, double x1, double y1, double x2, double y2, bool FLIP);
void objects_unselect(World *w);
void objects_unselect_no_refresh();
void objects_set_color_selected(World *w, unsigned long fg, unsigned long bg);
void objects_set_line_attr_selected(World *w, int lw, int ls);
void objects_delete_selected(World *w);
void objects_delete_selected_no_refresh();
void objects_undelete_selected(World *w);
void objects_highlight(World *w);
void objects_glue();
void objects_unglue();
void objects_anchor();
void objects_unanchor();
void objects_set_string(World *w, char *string);
void objects_dump();

// support for move & copy and other modes
GraphicObj* objects_object_selected_by_handle_at(double x, double y, bool no_if_glued = false);
GraphicObj* objects_object_selected_at(double x, double y, bool no_if_glued = false);
bool objects_something_selected_at(double x, double y, bool no_if_glued = false);
bool objects_duplicate_selected(bool no_if_glued = false);
void objects_translate_start(World *w, bool no_if_glued = false);
void objects_translate_change(World *w, double dx, double dy, bool no_if_glued = false);
void objects_translate_end(World *w, double dx, double dy, bool no_if_glued = false);

// permuting objects in the control lists
void objects_rotate_all(World *w);
void objects_selected_top(World *w);
void objects_selected_bottom(World *w);

// external interaction
list(GraphicObj_ptr)& objects_get_graphic_objects();
list(list_item)& objects_get_selected_objects();
void objects_install_created_objects(World *w, bool select, Obj_ptr, char *function = 0, char *module = 0);
void objects_install_created_objects(World *w, bool select, list(Obj_ptr) &, char *function = 0, char *module = 0);
void objects_install_created_objects_no_refresh(World *w, bool select, list(Obj_ptr) &, char *function = 0, char *module = 0);

list(Obj_ptr) extract_selected_objects(ObjType type, ObjType subtype = Generic_t);
list(Obj_ptr) extract_objects(ObjType type, ObjType subtype = Generic_t);
list(GraphicObj_ptr) extract_selected_graphic_objects(ObjType type, ObjType subtype = Generic_t);
list(GraphicObj_ptr) extract_graphic_objects(ObjType type, ObjType subtype = Generic_t);



#endif ~_OBJECTS_DEFINED








