/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * proximity.C - algoritmos para problemas de proximidade.
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include <values.h>
#include <math.h>
#include "proximity.h"
#include "ClosestPairDC.h"
#include "QuadEdge.h"
#include "voronoi.h"

/*
 **** variaveis para animacao ***/

static GraphicSegment2D *anim_s;

#define anim_segment	((Segment2D *)anim_s->obj)

void proximity_animation_init(World *w) {
    static bool FIRST = true;
    
    if (FIRST) {
      anim_s = new GraphicSegment2D();
      FIRST = false;
    }
    anim_s->set_all(w, FOREGROUND, BACKGROUND, 3, 0, false);
}

/*
 ********************************/

typedef QuadEdge DelaunayTriang;
typedef QuadEdge_ptr DelaunayTriang_ptr;

Segment2D_ptr closest_pair_bf(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
    list_item it_p, it_closest_p;
    Segment2D_ptr res = 0;		// O(n^2) algorithm

    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      proximity_animation_init(w);
    ANIMATION_END(w, animation_depth, reference_depth)
        
    if (points.size() > 1) {
      Point2D_ptr p1 = 0, p2 = 0;
      double menor_aux, menor = MAXDOUBLE;
      forall_list_items(it_p, points) {
        forall_list_items(it_closest_p, points) {

          ANIMATION_BEGIN(w, animation_depth, reference_depth)
            anim_segment->set(*points[it_p], *points[it_closest_p]);
            anim_s->draw(w, XorMode);
            frame_stop_mark(w);
            anim_s->draw(w, XorMode);
          ANIMATION_END(w, animation_depth, reference_depth)

          if (it_p != it_closest_p && (menor_aux = points[it_p]->e_distance_sqr(*points[it_closest_p])) < menor) {
          
            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              if (p1) {
              	anim_segment->set(*p1, *p2);
              	anim_s->draw(w, XorMode);
              }
            ANIMATION_END(w, animation_depth, reference_depth)

            p1 = points[it_p]; p2 = points[it_closest_p];

            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              anim_segment->set(*p1, *p2);
              anim_s->draw(w, XorMode);
              frame_stop_mark(w);
            ANIMATION_END(w, animation_depth, reference_depth)
            
            menor = menor_aux;
          }
        }
      }
      res = new Segment2D(*p1, *p2);
    }
    
    return res;
}

list(Segment2D_ptr) all_nearest_neighbors_bf(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
    list_item it_p, it_closest_p;
    list(Segment2D_ptr) res;		// O(n^2) algorithm
    
    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      proximity_animation_init(w);
    ANIMATION_END(w, animation_depth, reference_depth)
        
    if (points.size() > 1) {
      Point2D_ptr p1 = 0, p2 = 0;
      double menor_aux, menor;
      forall_list_items(it_p, points) {
        menor = MAXDOUBLE;
        forall_list_items(it_closest_p, points) {

          ANIMATION_BEGIN(w, animation_depth, reference_depth)
            anim_segment->set(*points[it_p], *points[it_closest_p]);
            anim_s->draw(w, XorMode);
            frame_stop_mark(w);
            anim_s->draw(w, XorMode);
          ANIMATION_END(w, animation_depth, reference_depth)

          if (it_p != it_closest_p && (menor_aux = points[it_p]->e_distance_sqr(*points[it_closest_p])) < menor) {

            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              if (p1) {
              	anim_segment->set(*p1, *p2);
              	anim_s->draw(w, XorMode);
              }
            ANIMATION_END(w, animation_depth, reference_depth)
            
            p1 = points[it_p]; p2 = points[it_closest_p];

            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              anim_segment->set(*p1, *p2);
              anim_s->draw(w, CopyMode);
              frame_stop_mark(w);
            ANIMATION_END(w, animation_depth, reference_depth)

            menor = menor_aux;
          }
        }
        
        ANIMATION_BEGIN(w, animation_depth, reference_depth)
          anim_segment->set(*p1, *p2);
          anim_s->draw(w, CopyMode);
        ANIMATION_END(w, animation_depth, reference_depth)
        
        res.push(new Segment2D(*p1, *p2));
      }
    }
    
    return res;
}




Segment2D_ptr closest_pair_dc(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
        Segment_ptr s;
	Segment2D_ptr res;
	InternalClosestPairDC algo;
	s = algo.find(points,w,animation_depth,reference_depth);
	res = s->s;
	delete s;
	
	return res;
	
}

list(Segment2D_ptr) all_nearest_neighbors_dc(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
    // O(n*log(n)) algorithm
    cerr << "Not implemented." << endl;
    list(Segment2D_ptr) LS;
    return LS;		  
}



// variables for recursive search
static QEdge_ptr rec_smallest_edge;
static double rec_smallest_length;
// for animation
static int rec_animation_depth;
static int rec_reference_depth;

QuadEdge_forall_df_BEGIN(recursive_find_smallest_edge, e)
    double smallest_aux;

    ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
      anim_segment->set(*(e->org()), *(e->dest()));
      anim_s->draw(rec_world, XorMode);
      frame_stop_mark(rec_world);
      anim_s->draw(rec_world, XorMode);
    ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)

    if ((smallest_aux = e->org()->e_distance_sqr(*(e->dest()))) < rec_smallest_length) {
    
      ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
        if (rec_smallest_edge) {
          anim_segment->set(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
          anim_s->draw(rec_world, XorMode);
        }
      ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)
      
      rec_smallest_length = smallest_aux;
      rec_smallest_edge = e;

      ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
        anim_segment->set(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
        anim_s->draw(rec_world, XorMode);
        frame_stop_mark(rec_world);
      ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)
      
    }    	
QuadEdge_forall_df_END(recursive_find_smallest_edge, e)

Segment2D_ptr closest_pair_vd(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
    Segment2D_ptr res = 0;
    DelaunayTriang *dt;
    
    if (points.size() > 1) {
      dt = delaunay_triangulation(points, w, animation_depth+1, reference_depth);
      if (dt->root()) {
      	rec_flag = *(dt->root()->flag());
      	rec_smallest_edge = 0; 
      	rec_smallest_length = MAXDOUBLE;
      	rec_world = w;
      	rec_animation_depth = animation_depth;
      	rec_reference_depth = reference_depth;
      	recursive_find_smallest_edge(dt->root());
      	res = new Segment2D(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
      }
      delete dt;
    }
    
    return res;
}

QuadEdge_forall_bf_BEGIN(recursive_find_all_nn, e)
    double smallest_aux;
    if (!already) {
      rec_smallest_length = MAXDOUBLE;
      rec_smallest_edge = 0;
      already = true;
    }

    ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
      anim_segment->set(*(eaux->org()), *(eaux->dest()));
      anim_s->draw(rec_world, XorMode);
      frame_stop_mark(rec_world);
      anim_s->draw(rec_world, XorMode);
    ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)

    if ((smallest_aux = eaux->org()->e_distance_sqr(*(eaux->dest()))) < rec_smallest_length) {

      ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
        if (rec_smallest_edge) {
          anim_segment->set(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
          anim_s->draw(rec_world, XorMode);
        }
      ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)
      
      rec_smallest_length = smallest_aux;
      rec_smallest_edge = eaux;

      ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
        anim_segment->set(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
        anim_s->draw(rec_world, XorMode);
        frame_stop_mark(rec_world);
      ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)
      
    }
    if (eaux->onext() == e) {

      ANIMATION_BEGIN(rec_world, rec_animation_depth, rec_reference_depth)
        anim_segment->set(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest()));
        anim_s->draw(rec_world, CopyMode);
      ANIMATION_END(rec_world, rec_animation_depth, rec_reference_depth)
      
      rec_Segment2D_list->push(new Segment2D(*(rec_smallest_edge->org()), *(rec_smallest_edge->dest())));
    }
QuadEdge_forall_bf_END(recursive_find_all_nn, e)

list(Segment2D_ptr) all_nearest_neighbors_vd(list(Point2D_ptr) &points, World *w, int animation_depth, int reference_depth) {
    list(Segment2D_ptr) res;
    DelaunayTriang *dt;
    
    if (points.size() > 1) {
      dt = delaunay_triangulation(points, w, animation_depth+1, reference_depth);
      if (dt->root()) {
      	rec_flag = *(dt->root()->oflag());
      	rec_Segment2D_list = new list(Segment2D_ptr);
      	rec_world = w;
      	rec_animation_depth = animation_depth;
      	rec_reference_depth = reference_depth;
      	recursive_find_all_nn(dt->root());
      	res = *rec_Segment2D_list;
      	delete rec_Segment2D_list;
      }
      delete dt;
    }
    
    return res;
}



