/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * random_polygon.C
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.01.92  13:40 	Cesar N. Gon 	  	Arquivo criado
 *
 */

#include "random_polygon.h"
#include <sys/time.h>


// prototypes

void randomize();

/*
 * implementacao dos metodos
 *
 */
 
bool Random_Polygon::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO)
{
    list_item it;
    
    forall(it, SO) {
	if (GO[it]->type() != Point2D_t) {
	    SO.init_iterator();
	    return false;
	}
    }
    return true;
}

ResultObjects convex_polygon::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Point2D_ptr) Points;
    Obj_ptr obj;
    list_item it;

    if (!SO.empty()) {	
	forall (it,SO) {
	    obj=GO[it]->obj;
	    if (obj->type()==Point2D_t)
	      Points.push((Point2D_ptr)obj);
	}
	Polygon_ptr P = new Polygon(Points); assert(P);
	randomize();
	P->convex();
	Result.push(P);
    }
    return Result;
}

Algorithm* convex_polygon_link() {
    return new convex_polygon();
}


ResultObjects generic_polygon::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Point2D_ptr) Points;
    Obj_ptr obj;
    list_item it;

    if (!SO.empty()) {	
	forall (it,SO) {
	    obj=GO[it]->obj;
	    if (obj->type()==Point2D_t)
	      Points.push((Point2D_ptr)obj);
	}
	Polygon_ptr P = new Polygon(Points); assert(P);
	randomize();
	P->permute();
	Result.push(P);
    }
    return Result;
}


Algorithm* generic_polygon_link() {
    return new generic_polygon();
}


ResultObjects starshaped_polygon::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Point2D_ptr) Points;
    Obj_ptr obj;
    list_item it;

    if (!SO.empty()) {	
	forall (it,SO) {
	    obj=GO[it]->obj;
	    if (obj->type()==Point2D_t)
	      Points.push((Point2D_ptr)obj);
	}
	Polygon_ptr P = new Polygon(Points); assert(P);
	randomize();
	P->permute();
	P->star_shaped(*(P->head()));
	Result.push(P);
    }
    return Result;
}

Algorithm* starshaped_polygon_link() {
    return new starshaped_polygon();
}


ResultObjects simple_polygon::binder(const SelectedObjects& SO, const GraphicObjects& GO, World *, int, int) {
    ResultObjects Result;
    list(Point2D_ptr) Points;
    Obj_ptr obj;
    list_item it;

    if (!SO.empty()) {	
	forall (it,SO) {
	    obj=GO[it]->obj;
	    if (obj->type()==Point2D_t)
	      Points.push((Point2D_ptr)obj);
	}
	Polygon_ptr P = new Polygon(Points); assert(P);
	randomize();
	P->simple();
	Result.push(P);
    }
    return Result;
}

Algorithm* simple_polygon_link() {
    return new simple_polygon();
}


// fun,c~oes auxiliares

    
/* inicializa o gerador de numeros aleatorios em funcao do tempo */
void randomize ()
{
    struct timeval now_tp;
    struct timezone now_tzp;
    gettimeofday (&now_tp,&now_tzp);
    srand ((int)now_tp.tv_sec);
}

